@@ -1,13 +1,9 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.component;
 
 import org.mule.DefaultMuleEvent;
@@ -174,7 +170,7 @@ public class BindingInvocationHandler implements InvocationHandler
     @Override
     public String toString()
     {
-        final StringBuffer sb = new StringBuffer();
+        final StringBuilder sb = new StringBuilder();
         sb.append("BindingInvocation");
         sb.append("{routers='").append(StringMessageUtils.toString(routers));
         sb.append('}');
