@@ -1,18 +1,14 @@
 /*
- * $Id$
- * -------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.config.spring.handlers;
 
 import org.mule.config.spring.parsers.generic.OrphanDefinitionParser;
 import org.mule.config.spring.parsers.specific.InvokerMessageProcessorDefinitionParser;
-import org.mule.config.spring.util.ParamReader;
+import org.mule.util.ParamReader;
 
 import java.io.IOException;
 import java.lang.reflect.Method;
