@@ -1,18 +1,17 @@
 /*
- * $Id: DefaultBeanAssembler.java 22816 2011-09-01 22:02:55Z mike.schilling $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.config.spring.parsers.assembly;
 
+import static org.springframework.util.SystemPropertyUtils.PLACEHOLDER_PREFIX;
+import static org.springframework.util.SystemPropertyUtils.PLACEHOLDER_SUFFIX;
+
 import org.mule.api.AnnotatedObject;
 import org.mule.api.MuleContext;
-import org.mule.config.spring.MuleApplicationContext;
+import org.mule.config.spring.MuleArtifactContext;
 import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
 import org.mule.config.spring.parsers.assembly.configuration.PropertyConfiguration;
 import org.mule.config.spring.parsers.assembly.configuration.SingleProperty;
@@ -133,7 +132,7 @@ public class DefaultBeanAssembler implements BeanAssembler
             }
             Object value = beanConfig.translateValue(oldName, oldValue);
             addAnnotationValue(beanDefinition.getPropertyValues(), name, value);
-            MuleContext muleContext = MuleApplicationContext.getCurrentMuleContext().get();
+            MuleContext muleContext = MuleArtifactContext.getCurrentMuleContext().get();
             if (muleContext != null)
             {
                 Map<QName, Set<Object>> annotations = muleContext.getConfigurationAnnotations();
@@ -148,9 +147,9 @@ public class DefaultBeanAssembler implements BeanAssembler
         }
         else
         {
-            if (logger.isWarnEnabled())
+            if (logger.isDebugEnabled())
             {
-                logger.warn("Cannot assign "+beanDefinition.getBeanClass()+" to "+AnnotatedObject.class);
+                logger.debug("Cannot assign "+beanDefinition.getBeanClass()+" to "+AnnotatedObject.class);
             }
         }
     }
@@ -583,5 +582,32 @@ public class DefaultBeanAssembler implements BeanAssembler
         return false;
     }
 
+    @Override
+    public String resolvePlaceholder(String value)
+    {
+        MuleContext context = MuleArtifactContext.getCurrentMuleContext().get();
+        String deploymentPropertyResolvedValue = getDeploymentPropertyResolvedValue(value, context);
+        
+        return deploymentPropertyResolvedValue;        
+    }
 
+    private String getDeploymentPropertyResolvedValue(String value, MuleContext context)
+    {
+        String deploymentPropertyResolvedValue = null;
+        if (isPlaceholder(value))
+        {
+            deploymentPropertyResolvedValue = (String) context.getDeploymentProperties().get(extractName(value));
+        }
+        return deploymentPropertyResolvedValue;
+    }
+
+    private String extractName(String value)
+    {
+        return value.substring(value.indexOf(PLACEHOLDER_PREFIX) + 2, value.indexOf(PLACEHOLDER_SUFFIX));
+    }
+
+    private boolean isPlaceholder(String value)
+    {
+        return value.indexOf(PLACEHOLDER_PREFIX) != -1 && value.indexOf(PLACEHOLDER_SUFFIX) != -1 && value.indexOf(PLACEHOLDER_PREFIX) < value.indexOf(PLACEHOLDER_SUFFIX);
+    }
 }
