@@ -1,8 +1,5 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
@@ -22,7 +19,12 @@ public class ThreadingProfileDefinitionParser extends ChildDefinitionParser
 
     public ThreadingProfileDefinitionParser(String propertyName, String defaults)
     {
-        super(propertyName, ChainedThreadingProfile.class);
+        this(propertyName, defaults, ChainedThreadingProfile.class);
+    }
+
+    public ThreadingProfileDefinitionParser(String propertyName, String defaults, Class<? extends ThreadingProfile> threadingProfileClass)
+    {
+        super(propertyName, threadingProfileClass);
         addMapping("poolExhaustedAction", ThreadingProfile.POOL_EXHAUSTED_ACTIONS);
         registerPostProcessor(new ConstructorReference(defaults));
     }
