@@ -1,13 +1,9 @@
 /*
- * $Id: BridgeBuilder.java 19920 2010-10-15 00:33:14Z ddossot $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.construct.builder;
 
 import java.util.Arrays;
@@ -60,15 +56,15 @@ public class BridgeBuilder extends
         return this;
     }
 
-    public BridgeBuilder inboundTransformers(Transformer... transformers)
+    public BridgeBuilder transformers(Transformer... transformers)
     {
-        this.inboundTransformers = Arrays.asList((MessageProcessor[]) transformers);
+        this.transformers = Arrays.asList((MessageProcessor[]) transformers);
         return this;
     }
 
-    public BridgeBuilder inboundResponseTransformers(Transformer... responseTransformers)
+    public BridgeBuilder responseTransformers(Transformer... responseTransformers)
     {
-        this.inboundResponseTransformers = Arrays.asList((MessageProcessor[]) responseTransformers);
+        this.responseTransformers = Arrays.asList((MessageProcessor[]) responseTransformers);
         return this;
     }
 
@@ -77,9 +73,8 @@ public class BridgeBuilder extends
     {
         if (transacted)
         {
-            MuleTransactionConfig transactionConfig = new MuleTransactionConfig();
+            MuleTransactionConfig transactionConfig = new MuleTransactionConfig(TransactionConfig.ACTION_BEGIN_OR_JOIN);
             transactionConfig.setMuleContext(muleContext);
-            transactionConfig.setAction(TransactionConfig.ACTION_BEGIN_OR_JOIN);
             endpointBuilder.setTransactionConfig(transactionConfig);
         }
     }
@@ -89,9 +84,8 @@ public class BridgeBuilder extends
     {
         if (transacted)
         {
-            MuleTransactionConfig transactionConfig = new MuleTransactionConfig();
+            MuleTransactionConfig transactionConfig = new MuleTransactionConfig(TransactionConfig.ACTION_ALWAYS_JOIN);
             transactionConfig.setMuleContext(muleContext);
-            transactionConfig.setAction(TransactionConfig.ACTION_ALWAYS_JOIN);
             endpointBuilder.setTransactionConfig(transactionConfig);
         }
     }
@@ -107,7 +101,8 @@ public class BridgeBuilder extends
             setTransactionFactoriesIfNeeded(inboundEndpoint, outboundEndpoint);
         }
 
-        return new Bridge(name, muleContext, inboundEndpoint, outboundEndpoint, exchangePattern, transacted);
+        return new Bridge(name, muleContext, inboundEndpoint, outboundEndpoint, transformers,
+            responseTransformers, exchangePattern, transacted);
     }
 
     private void setTransactionFactoriesIfNeeded(InboundEndpoint inboundEndpoint,
