@@ -14,6 +14,7 @@ import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
@@ -28,11 +29,12 @@ class Policy
 
     // map from event to set of senders
     private Map<Class<? extends ServerNotification>, Collection<Sender>> eventToSenders = new HashMap<Class<? extends ServerNotification>, Collection<Sender>>();
+    private Map<Class<? extends ServerNotification>, Collection<Sender>> concreteEventToSenders = new HashMap<Class<? extends ServerNotification>, Collection<Sender>>();
 
     // these are cumulative - set values should never change, they are just a cache of known info
     // they are co and contra-variant wrt to exact event type (see code below).
-    private ConcurrentMap knownEventsExact = new ConcurrentHashMap();
-    private ConcurrentMap knownEventsSuper = new ConcurrentHashMap();
+    private ConcurrentMap<Class, Boolean> knownEventsExact = new ConcurrentHashMap<>();
+    private ConcurrentMap<Class, Boolean> knownEventsSuper = new ConcurrentHashMap<>();
 
     /**
      * For each listener, we check each interface and see what events can be delivered.
@@ -103,13 +105,14 @@ class Policy
         if (null != notification)
         {
             Class notfnClass = notification.getClass();
+            Boolean eventKnown = knownEventsExact.get(notfnClass);
             // search if we don't know about this event, or if we do know it is used
-            if (!knownEventsExact.containsKey(notfnClass))
+            if (eventKnown == null)
             {
                 boolean found = doDispatch(notification, notfnClass);
                 knownEventsExact.put(notfnClass, Boolean.valueOf(found));
             }
-            else if (((Boolean) knownEventsExact.get(notfnClass)).booleanValue())
+            else if (eventKnown.booleanValue())
             {
                 boolean found = doDispatch(notification, notfnClass);
                 // reduce contention on the map by not writing the same value over and over again.
@@ -124,20 +127,56 @@ class Policy
     protected boolean doDispatch(ServerNotification notification, Class<? extends ServerNotification> notfnClass)
     {
         boolean found = false;
-        for (Class<? extends ServerNotification> event : eventToSenders.keySet())
+        
+        // Optimization to avoid iterating the eventToSenders map each time a notification is fired
+        Collection<Sender> senders = concreteEventToSenders.get(notfnClass);
+        if(senders != null)
         {
-            if (event.isAssignableFrom(notfnClass))
+            found = true;
+            dispatchToSenders(notification, senders);
+        }
+        else
+        {
+            senders = concreteEventToSenders.get(notfnClass);
+            if(senders != null)
             {
-                found = true;
-                for (Sender sender : eventToSenders.get(event))
+                dispatchToSenders(notification, senders);
+            }
+            else
+            {
+                senders = new ArrayList<Sender>();
+                for (Entry<Class<? extends ServerNotification>, Collection<Sender>> event : eventToSenders.entrySet())
                 {
-                    sender.dispatch(notification);
+                    if (event.getKey().isAssignableFrom(notfnClass))
+                    {
+                        found = true;
+                        senders.addAll(event.getValue());
+                        dispatchToSenders(notification, senders);
+                    }
+                }
+                if(!concreteEventToSenders.containsKey(notfnClass)) {
+                    concreteEventToSenders.put(notfnClass, senders);
                 }
             }
         }
         return found;
     }
 
+    private void dispatchToSenders(ServerNotification notification, Collection<Sender> senders)
+    {
+        for (Sender sender : senders)
+        {
+            try
+            {
+                sender.dispatch(notification);
+            }
+            catch (Exception e)
+            {
+                // Exceptions from listeners do not affect the notification processing
+            }
+        }
+    }
+
     /**
      * This returns a very "conservative" value - it is true if the notification or any subclass would be
      * accepted.  So if it returns false then you can be sure that there is no need to send the
@@ -149,18 +188,22 @@ class Policy
      */
     boolean isNotificationEnabled(Class notfnClass)
     {
-        if (!knownEventsSuper.containsKey(notfnClass))
+        Boolean knownSuper = knownEventsSuper.get(notfnClass);
+        if (knownSuper == null)
         {
             boolean found = false;
             // this is exhaustive because we initialise to include all events handled.
-            for (Iterator events = knownEventsSuper.keySet().iterator(); events.hasNext() && !found;)
+            for (Iterator<Class> events = knownEventsSuper.keySet().iterator(); events.hasNext() && !found;)
             {
-                Class event = (Class) events.next();
-                found = ((Boolean) knownEventsSuper.get(event)).booleanValue() && notfnClass.isAssignableFrom(event);
+                Class event = events.next();
+                found = knownEventsSuper.get(event).booleanValue() && notfnClass.isAssignableFrom(event);
             }
-            knownEventsSuper.put(notfnClass, Boolean.valueOf(found));
+            knownSuper = Boolean.valueOf(found);
+            knownEventsSuper.put(notfnClass, knownSuper);
         }
-        if (!knownEventsExact.containsKey(notfnClass))
+        
+        Boolean knownExact = knownEventsExact.get(notfnClass);
+        if (knownExact == null)
         {
             boolean found = false;
             for (Iterator events = eventToSenders.keySet().iterator(); events.hasNext() && !found;)
@@ -168,11 +211,11 @@ class Policy
                 Class event = (Class) events.next();
                 found = event.isAssignableFrom(notfnClass);
             }
-            knownEventsExact.put(notfnClass, Boolean.valueOf(found));
-
+            knownExact = Boolean.valueOf(found);
+            knownEventsExact.put(notfnClass, knownExact);
         }
-        return ((Boolean) knownEventsSuper.get(notfnClass)).booleanValue()
-                || ((Boolean) knownEventsExact.get(notfnClass)).booleanValue();
+        
+        return knownSuper || knownExact;
     }
 
 }
