@@ -1,15 +1,12 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.context.notification;
 
+import org.mule.api.context.notification.AbstractBlockingServerEvent;
 import org.mule.api.context.notification.BlockingServerEvent;
 import org.mule.api.context.notification.ServerNotification;
 import org.mule.api.service.Service;
@@ -20,7 +17,7 @@ import org.mule.api.service.Service;
  * service.
  */
 @Deprecated
-public class ServiceNotification extends ServerNotification implements BlockingServerEvent
+public class ServiceNotification extends AbstractBlockingServerEvent
 {
     /**
      * Serial version
