@@ -1,27 +1,34 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
 package org.mule.exception;
 
+import static java.lang.Boolean.getBoolean;
+import static org.mule.api.config.MuleProperties.DISABLE_ERROR_COUNT_ON_ERROR_NOTIFICATION_DISABLED;
+import static org.mule.config.i18n.MessageFactory.createStaticMessage;
+
+import org.mule.DefaultMuleEvent;
 import org.mule.VoidMuleEvent;
 import org.mule.api.MessagingException;
 import org.mule.api.MuleContext;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
+import org.mule.api.config.MuleProperties;
 import org.mule.api.construct.FlowConstruct;
+import org.mule.api.exception.MessagingExceptionHandler;
 import org.mule.api.exception.MessagingExceptionHandlerAcceptor;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.processor.MessageProcessor;
 import org.mule.api.processor.MessageProcessorChain;
+import org.mule.api.transport.NonBlockingReplyToHandler;
+import org.mule.config.i18n.CoreMessages;
 import org.mule.context.notification.ExceptionStrategyNotification;
 import org.mule.management.stats.FlowConstructStatistics;
 import org.mule.message.DefaultExceptionPayload;
+import org.mule.processor.AbstractRequestResponseMessageProcessor;
 import org.mule.processor.chain.DefaultMessageProcessorChainBuilder;
 import org.mule.routing.requestreply.ReplyToPropertyRequestReplyReplier;
 import org.mule.transaction.TransactionCoordination;
@@ -33,55 +40,91 @@ public abstract class TemplateMessagingExceptionStrategy extends AbstractExcepti
     private MessageProcessor replyToMessageProcessor = new ReplyToPropertyRequestReplyReplier();
     private String when;
     private boolean handleException;
+    private boolean disableErrorCountOnErrorNotificationsDisabled = getBoolean(DISABLE_ERROR_COUNT_ON_ERROR_NOTIFICATION_DISABLED);
 
     final public MuleEvent handleException(Exception exception, MuleEvent event)
     {
         try
         {
-            muleContext.getNotificationManager().fireNotification(new ExceptionStrategyNotification(event, ExceptionStrategyNotification.PROCESS_START));
-            FlowConstruct flowConstruct = event.getFlowConstruct();
-            fireNotification(exception);
-            logException(exception);
-            processStatistics(event);
-            event.getMessage().setExceptionPayload(new DefaultExceptionPayload(exception));
-            event = beforeRouting(exception, event);
-            event = route(event, exception);
-            processOutboundRouterStatistics(flowConstruct);
-            event = afterRouting(exception, event);
-            markExceptionAsHandledIfRequired(exception);
-            if (event != null && !VoidMuleEvent.getInstance().equals(event))
-            {
-                processReplyTo(event, exception);
-                closeStream(event.getMessage());
-                nullifyExceptionPayloadIfRequired(event);
-            }
-            return event;
+            return new ExceptionMessageProcessor(exception, muleContext, event.getFlowConstruct()).process(event);
         }
-        catch (Exception e)
+        catch (MuleException e)
         {
-            MessagingException messagingException;
-            if (e instanceof MessagingException)
+            throw new RuntimeException(e);
+        }
+    }
+
+    private class ExceptionMessageProcessor extends AbstractRequestResponseMessageProcessor{
+
+        private Exception exception;
+
+        public ExceptionMessageProcessor(Exception exception, MuleContext muleContext, FlowConstruct flowConstruct)
+        {
+            this.exception = exception;
+            setMuleContext(muleContext);
+            setFlowConstruct(flowConstruct);
+        }
+
+        @Override
+        protected MuleEvent processRequest(MuleEvent request) throws MuleException
+        {
+            if (!handleException && request.getReplyToHandler() instanceof NonBlockingReplyToHandler)
             {
-                messagingException = (MessagingException) e;
+                request = new DefaultMuleEvent(request, request.getFlowConstruct(), null, null, true);
             }
-            else 
+            muleContext.getNotificationManager().fireNotification(new ExceptionStrategyNotification(request, ExceptionStrategyNotification.PROCESS_START));
+            fireNotification(exception);
+            logException(exception, request);
+            processStatistics(request);
+            request.getMessage().setExceptionPayload(new DefaultExceptionPayload(exception));
+            markExceptionAsHandledIfRequired(exception);
+            return beforeRouting(exception, request);
+        }
+
+        @Override
+        protected MuleEvent processResponse(MuleEvent response, MuleEvent request) throws MuleException
+        {
+            processOutboundRouterStatistics(flowConstruct);
+            response = afterRouting(exception, response);
+            if (response != null && !VoidMuleEvent.getInstance().equals(response))
             {
-                messagingException = new MessagingException(event, e); 
+                // Only process reply-to if non-blocking is not enabled. Checking the exchange pattern is not sufficient
+                // because JMS inbound endpoints for example use a REQUEST_RESPONSE exchange pattern and async processing.
+                if (!(request.isAllowNonBlocking() && request.getReplyToHandler() instanceof NonBlockingReplyToHandler))
+                {
+                    processReplyTo(response, exception);
+                }
+                closeStream(response.getMessage());
+                nullifyExceptionPayloadIfRequired(response);
             }
+            return response;
+        }
+
+        @Override
+        protected MuleEvent processNext(MuleEvent event) throws MuleException
+        {
+            return route(event, exception);
+        }
+
+        @Override
+        protected MuleEvent processCatch(MuleEvent event, MessagingException exception) throws MessagingException
+        {
             try
             {
                 logger.error("Exception during exception strategy execution");
-                logException(e);
+                doLogException(exception);
                 TransactionCoordination.getInstance().rollbackCurrentTransaction();
             }
             catch (Exception ex)
             {
                 //Do nothing
             }
-            event.getMessage().setExceptionPayload(new DefaultExceptionPayload(messagingException));
+            event.getMessage().setExceptionPayload(new DefaultExceptionPayload(exception));
             return event;
         }
-        finally
+
+        @Override
+        protected void processFinally(MuleEvent event, MessagingException exception)
         {
             muleContext.getNotificationManager().fireNotification(new ExceptionStrategyNotification(event, ExceptionStrategyNotification.PROCESS_END));
         }
@@ -94,7 +137,7 @@ public abstract class TemplateMessagingExceptionStrategy extends AbstractExcepti
             markExceptionAsHandled(exception);
         }
     }
-    
+
     protected void markExceptionAsHandled(Exception exception)
     {
         if (exception instanceof MessagingException)
@@ -126,31 +169,36 @@ public abstract class TemplateMessagingExceptionStrategy extends AbstractExcepti
     private void processStatistics(MuleEvent event)
     {
         FlowConstructStatistics statistics = event.getFlowConstruct().getStatistics();
-        if (statistics != null && statistics.isEnabled())
+        if (statistics != null && statistics.isEnabled() && mustCountErrorInStatistics())
         {
             statistics.incExecutionError();
         }
     }
 
-    protected MuleEvent route(MuleEvent event, Exception t)
+    private boolean mustCountErrorInStatistics()
+    {
+        return enableNotifications || !disableErrorCountOnErrorNotificationsDisabled;
+    }
+
+    protected MuleEvent route(MuleEvent event, Exception t) throws MessagingException
     {
         if (!getMessageProcessors().isEmpty())
         {
             try
             {
                 event.getMessage().setExceptionPayload(new DefaultExceptionPayload(t));
-                MuleEvent result = configuredMessageProcessors.process(event);                
+                MuleEvent result = configuredMessageProcessors.process(event);
                 return result;
             }
             catch (Exception e)
             {
                 logFatal(event, e);
+                throw new MessagingException(createStaticMessage("There was an error processing the catch strategy"), event, e);
             }
         }
         return event;
     }
 
-
     @Override
     protected void doInitialise(MuleContext muleContext) throws InitialisationException
     {
@@ -206,6 +254,12 @@ public abstract class TemplateMessagingExceptionStrategy extends AbstractExcepti
         return event;
     }
 
+    @Override
+    public void setMessagingExceptionHandler(MessagingExceptionHandler messagingExceptionHandler)
+    {
+        return;
+    }
+
     public void setHandleException(boolean handleException)
     {
         this.handleException = handleException;
