@@ -22,6 +22,8 @@ import org.mule.api.lifecycle.Initialisable;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.el.mvel.MVELExpressionLanguage;
+import org.mule.transformer.types.TypedValue;
+import org.mule.transport.NullPayload;
 import org.mule.util.StringUtils;
 import org.mule.util.TemplateParser;
 
@@ -49,7 +51,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
      */
     protected static transient final Log logger = LogFactory.getLog(DefaultExpressionManager.class);
 
-    private static final String OBJECT_FOR_ENRICHMENT = "__object_for_enrichment";
+    public static final String OBJECT_FOR_ENRICHMENT = "__object_for_enrichment";
 
     // default style parser
     private TemplateParser parser = TemplateParser.createMuleStyleParser();
@@ -205,7 +207,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
     public Object evaluate(String expression, MuleEvent event, boolean failIfNull)
         throws ExpressionRuntimeException
     {
-        expression = preProcessExpression(expression);
+        expression = removeExpressionMarker(expression);
         if (isEvaluatorExpression(expression))
         {
             return evaluate(expression, event.getMessage(), failIfNull);
@@ -218,7 +220,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
 
     public Object evaluate(String expression, MuleMessage message, boolean failIfNull)
     {
-        expression = preProcessExpression(expression);
+        expression = removeExpressionMarker(expression);
         if (isEvaluatorExpression(expression))
         {
             String[] parts = expression.split(":", 2);
@@ -233,7 +235,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
     public void enrich(String expression, MuleMessage message, Object object)
         throws ExpressionRuntimeException
     {
-        expression = preProcessExpression(expression);
+        expression = removeExpressionMarker(expression);
         if (isEvaluatorExpression(expression))
         {
             String[] parts = expression.split(":", 2);
@@ -241,15 +243,31 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         }
         else
         {
+
             expressionLanguage.evaluate(createEnrichmentExpression(expression), message,
-                Collections.singletonMap(OBJECT_FOR_ENRICHMENT, object));
+                                        Collections.singletonMap(OBJECT_FOR_ENRICHMENT, object));
+        }
+    }
+
+    @Override
+    public void enrichTyped(String expression, MuleMessage message, TypedValue object)
+    {
+        expression = removeExpressionMarker(expression);
+        if (isEvaluatorExpression(expression))
+        {
+            String[] parts = expression.split(":", 2);
+            enrich(parts[1], parts[0], message, object.getValue());
+        }
+        else
+        {
+            expressionLanguage.enrich(createEnrichmentExpression(expression), message, object);
         }
     }
 
     @Override
     public void enrich(String expression, MuleEvent event, Object object)
     {
-        expression = preProcessExpression(expression);
+        expression = removeExpressionMarker(expression);
         if (isEvaluatorExpression(expression))
         {
             enrich(expression, event.getMessage(), object);
@@ -258,7 +276,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         {
             expression = createEnrichmentExpression(expression);
             expressionLanguage.evaluate(expression, event,
-                Collections.singletonMap(OBJECT_FOR_ENRICHMENT, object));
+                                        Collections.singletonMap(OBJECT_FOR_ENRICHMENT, object));
         }
     }
 
@@ -297,25 +315,54 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         {
             return expressionLanguage.evaluate(expression, message);
         }
+        ExpressionEvaluator extractor = getExpressionEvaluator(evaluator);
+        Object result = extractor.evaluate(expression, message);
+        checkRequiredValue(expression, evaluator, failIfNull, result);
+        return result;
+    }
+
+    @Override
+    public TypedValue evaluateTyped(String expression, String evaluator, MuleMessage message, boolean failIfNull)
+            throws ExpressionRuntimeException
+    {
+        if (evaluator == null)
+        {
+            return expressionLanguage.evaluateTyped(expression, message);
+        }
+        ExpressionEvaluator extractor = getExpressionEvaluator(evaluator);
+
+        TypedValue result = extractor.evaluateTyped(expression, message);
+
+        checkRequiredValue(expression, evaluator, failIfNull, result.getValue());
+
+        return result;
+    }
+
+    private ExpressionEvaluator getExpressionEvaluator(String evaluator)
+    {
         ExpressionEvaluator extractor = (ExpressionEvaluator) evaluators.get(evaluator);
+
         if (extractor == null)
         {
             throw new IllegalArgumentException(CoreMessages.expressionEvaluatorNotRegistered(evaluator)
-                .getMessage());
+                                                       .getMessage());
         }
-        Object result = extractor.evaluate(expression, message);
-        // TODO Handle empty collections || (result instanceof Collection && ((Collection)result).size()==0)
-        if (failIfNull && (result == null))
+
+        return extractor;
+    }
+
+    private void checkRequiredValue(String expression, String evaluator, boolean failIfNull, Object value)
+    {
+        if (failIfNull && (value == null))
         {
             throw new RequiredValueException(CoreMessages.expressionEvaluatorReturnedNull(evaluator,
-                expression));
+                                                                                          expression));
         }
         if (logger.isDebugEnabled())
         {
             logger.debug(MessageFormat.format("Result of expression: {0}:{1} is: {2}", evaluator, expression,
-                result));
+                                              value));
         }
-        return result;
     }
 
     public boolean evaluateBoolean(String expression, String evaluator, MuleMessage message)
@@ -377,7 +424,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
                                      boolean nonBooleanReturnsTrue,
                                      String expression)
     {
-        if (result == null)
+        if (result == null || NullPayload.getInstance().equals(result))
         {
             return nullReturnsTrue;
         }
@@ -520,6 +567,21 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         return (string.contains(DEFAULT_EXPRESSION_PREFIX));
     }
 
+    @Override
+    public TypedValue evaluateTyped(String expression, MuleMessage message)
+    {
+        expression = removeExpressionMarker(expression);
+        if (isEvaluatorExpression(expression))
+        {
+            String[] parts = expression.split(":", 2);
+            return evaluateTyped(parts[1], parts[0], message, false);
+        }
+        else
+        {
+            return expressionLanguage.evaluateTyped(expression, message);
+        }
+    }
+
     /**
      * Determines if the expression is valid or not. This method will validate a single expression or
      * expressions embedded in a string. the expression must be well formed i.e. #[bean:user]
@@ -537,6 +599,10 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         catch (InvalidExpressionException e)
         {
             logger.warn(e.getMessage());
+            if (logger.isDebugEnabled())
+            {
+            	logger.debug("Expression '" + expression + "' is invalid.", e);
+            }
             return false;
         }
     }
@@ -613,7 +679,7 @@ public class DefaultExpressionManager implements ExpressionManager, MuleContextA
         return evaluateBoolean(expression, event, false, false);
     }
 
-    protected String preProcessExpression(String expression)
+    public static String removeExpressionMarker(String expression)
     {
         if (expression == null)
         {
