@@ -17,6 +17,7 @@ import org.mule.api.MuleRuntimeException;
 import org.mule.api.construct.FlowConstruct;
 import org.mule.api.construct.FlowConstructAware;
 import org.mule.api.interceptor.Interceptor;
+import org.mule.api.transport.NonBlockingReplyToHandler;
 import org.mule.api.transport.ReplyToHandler;
 import org.mule.management.stats.ProcessingTime;
 import org.mule.processor.AbstractRequestResponseMessageProcessor;
@@ -70,7 +71,7 @@ public abstract class AbstractEnvelopeInterceptor extends AbstractRequestRespons
     {
         final long startTime = System.currentTimeMillis();
         final ProcessingTime time = event.getProcessingTime();
-        MuleEvent responseEvent = event;
+        MuleEvent responseEvent = before(event);
 
         final ReplyToHandler originalReplyToHandler = event.getReplyToHandler();
         responseEvent = new DefaultMuleEvent(event, new ResponseReplyToHandler(originalReplyToHandler, time, startTime));
@@ -84,11 +85,12 @@ public abstract class AbstractEnvelopeInterceptor extends AbstractRequestRespons
             {
                 responseEvent = processResponse(responseEvent, event);
             }
+            
+            responseEvent = after(responseEvent);
         }
-        catch (Exception exception)
+        finally
         {
-            last(responseEvent, time, startTime, true);
-            throw exception;
+            responseEvent = last(responseEvent, time, startTime, true);
         }
         return responseEvent;
     }
@@ -98,7 +100,7 @@ public abstract class AbstractEnvelopeInterceptor extends AbstractRequestRespons
         this.flowConstruct = flowConstruct;
     }
 
-    class ResponseReplyToHandler implements ReplyToHandler
+    class ResponseReplyToHandler implements NonBlockingReplyToHandler
     {
 
         private final ReplyToHandler originalReplyToHandler;
