@@ -1,13 +1,9 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.module.boot;
 
 import java.net.URL;
@@ -22,6 +18,10 @@ import org.apache.commons.logging.LogFactory;
  */
 public class MuleSystemClassLoader extends URLClassLoader
 {
+    static
+    {
+        registerAsParallelCapable();
+    }
 
     protected transient Log logger = LogFactory.getLog(getClass());
 
