@@ -6,6 +6,11 @@
  */
 package org.mule.module.http.api;
 
+import static com.google.common.collect.ImmutableSet.of;
+import static org.mule.module.http.api.HttpConstants.RequestProperties.HTTP_PREFIX;
+
+import java.util.Set;
+
 /**
  * General purpose HTTP constants
  */
@@ -17,6 +22,8 @@ public abstract class HttpConstants
         POST, GET, PUT, OPTIONS, HEAD, DELETE;
     }
 
+    public static final Set<String> IDEMPOTENT_METHODS = of("GET", "PUT", "DELETE", "HEAD", "OPTIONS", "TRACE");
+
     public static enum Protocols
     {
         HTTP("http", 80), HTTPS("https", 443);
@@ -57,12 +64,19 @@ public abstract class HttpConstants
         public static final String HTTP_LISTENER_PATH = HTTP_PREFIX + "listener.path";
         public static final String HTTP_SCHEME = HTTP_PREFIX + "scheme";
         public static final String HTTP_CLIENT_CERTIFICATE = HTTP_PREFIX + "client.cert";
+        public static final String HTTP_CLIENT_SSL_SESSION = HTTP_PREFIX + "ssl.session";
     }
 
     public static final class ResponseProperties
     {
         public static final String HTTP_STATUS_PROPERTY = RequestProperties.HTTP_STATUS_PROPERTY;
-        public static final String HTTP_REASON_PROPERTY = RequestProperties.HTTP_PREFIX + "reason";
+        public static final String HTTP_REASON_PROPERTY = HTTP_PREFIX + "reason";
+    }
+
+    public static final class HttpProperties
+    {
+        public static String GRIZZLY_MEMORY_MANAGER_SYSTEM_PROPERTY = "org.glassfish.grizzly.DEFAULT_MEMORY_MANAGER";
+        public static String COMPATIBILITY_IGNORE_CORRELATION_ID = "mule.http.server.ignoreCorrelationId";
     }
 
     public static final String ALL_INTERFACES_IP = "0.0.0.0";
@@ -75,6 +89,7 @@ public abstract class HttpConstants
         OK(200, "OK"),
         CREATED(201, "Created"),
         ACCEPTED(202, "Accepted"),
+        DROPPED(202, "Dropped"),
         NON_AUTHORITATIVE_INFORMATION(203, "Non Authoritative Information"),
         NO_CONTENT(204, "No Content"),
         RESET_CONTENT(205, "Reset Content"),
@@ -130,6 +145,19 @@ public abstract class HttpConstants
         {
             return reasonPhrase;
         }
+
+        public static String getReasonPhraseForStatusCode(int statusCode)
+        {
+            for(HttpStatus httpStatus : HttpStatus.values())
+            {
+                if (httpStatus.getStatusCode() == statusCode)
+                {
+                    return httpStatus.getReasonPhrase();
+                }
+            }
+
+            return null;
+        }
     }
 
 }
