@@ -6,12 +6,23 @@
  */
 package org.mule.module.http.internal.listener;
 
+import static org.mule.api.transformer.DataType.BYTE_ARRAY_DATA_TYPE;
+import static org.mule.module.http.api.HttpConstants.HttpStatus.NO_CONTENT;
+import static org.mule.module.http.api.HttpConstants.HttpStatus.getReasonPhraseForStatusCode;
 import static org.mule.module.http.api.HttpConstants.RequestProperties.HTTP_PREFIX;
+import static org.mule.module.http.api.HttpConstants.RequestProperties.HTTP_STATUS_PROPERTY;
+import static org.mule.module.http.api.HttpConstants.RequestProperties.HTTP_VERSION_PROPERTY;
+import static org.mule.module.http.api.HttpConstants.ResponseProperties.HTTP_REASON_PROPERTY;
+import static org.mule.module.http.api.HttpHeaders.Names.CONNECTION;
 import static org.mule.module.http.api.HttpHeaders.Names.CONTENT_LENGTH;
 import static org.mule.module.http.api.HttpHeaders.Names.TRANSFER_ENCODING;
 import static org.mule.module.http.api.HttpHeaders.Values.CHUNKED;
+import static org.mule.module.http.api.HttpHeaders.Values.CLOSE;
 import static org.mule.module.http.api.requester.HttpStreamingType.ALWAYS;
 import static org.mule.module.http.api.requester.HttpStreamingType.AUTO;
+import static org.mule.module.http.internal.domain.HttpProtocol.HTTP_0_9;
+import static org.mule.module.http.internal.domain.HttpProtocol.HTTP_1_0;
+
 import org.mule.api.MessagingException;
 import org.mule.api.MuleContext;
 import org.mule.api.MuleEvent;
@@ -20,7 +31,7 @@ import org.mule.api.context.MuleContextAware;
 import org.mule.api.lifecycle.Initialisable;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.transformer.DataType;
-import org.mule.module.http.api.HttpConstants;
+import org.mule.api.transport.OutputHandler;
 import org.mule.module.http.api.HttpHeaders;
 import org.mule.module.http.api.requester.HttpStreamingType;
 import org.mule.module.http.internal.HttpMessageBuilder;
@@ -30,8 +41,10 @@ import org.mule.module.http.internal.ParameterMap;
 import org.mule.module.http.internal.domain.ByteArrayHttpEntity;
 import org.mule.module.http.internal.domain.EmptyHttpEntity;
 import org.mule.module.http.internal.domain.HttpEntity;
+import org.mule.module.http.internal.domain.HttpProtocol;
 import org.mule.module.http.internal.domain.InputStreamHttpEntity;
 import org.mule.module.http.internal.domain.MultipartHttpEntity;
+import org.mule.module.http.internal.domain.OutputHandlerHttpEntity;
 import org.mule.module.http.internal.domain.response.HttpResponse;
 import org.mule.module.http.internal.multipart.HttpMultipartEncoder;
 import org.mule.module.http.internal.multipart.HttpPartDataSource;
@@ -39,13 +52,13 @@ import org.mule.transformer.types.MimeTypes;
 import org.mule.transport.NullPayload;
 import org.mule.util.AttributeEvaluator;
 import org.mule.util.DataTypeUtils;
-import org.mule.util.IOUtils;
 import org.mule.util.NumberUtils;
 import org.mule.util.UUID;
 
 import java.io.InputStream;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.Set;
 
@@ -62,12 +75,13 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
     private String reasonPhrase;
     private boolean disablePropertiesAsHeaders = false;
     private HttpStreamingType responseStreaming = AUTO;
-    private boolean multipartEntityWithNoMultipartContentyTypeWarned;
-    private boolean mapPayloadButNoUrlEncodedContentyTypeWarned;
+    private boolean multipartEntityWithNoMultipartContentTypeWarned;
+    private boolean mapPayloadButNoUrlEncodedContentTypeWarned;
     private AttributeEvaluator statusCodeEvaluator;
     private AttributeEvaluator reasonPhraseEvaluator;
     private MuleContext muleContext;
 
+    @Override
     public void initialise() throws InitialisationException
     {
         super.initialise();
@@ -80,7 +94,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
         reasonPhraseEvaluator = new AttributeEvaluator(reasonPhrase).initialize(muleContext.getExpressionManager());
     }
 
-    public HttpResponse build(org.mule.module.http.internal.domain.response.HttpResponseBuilder httpResponseBuilder, MuleEvent event) throws MessagingException
+    public HttpResponse build(org.mule.module.http.internal.domain.response.HttpResponseBuilder httpResponseBuilder, MuleEvent event, String httpVersion) throws MessagingException
     {
         final HttpResponseHeaderBuilder httpResponseHeaderBuilder = new HttpResponseHeaderBuilder();
         final Set<String> outboundProperties = event.getMessage().getOutboundPropertyNames();
@@ -89,7 +103,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
         {
             for (String outboundPropertyName : outboundProperties)
             {
-                if (!outboundPropertyName.startsWith(HTTP_PREFIX))
+                if (isNotIgnoredProperty(outboundPropertyName))
                 {
                     final Object outboundPropertyValue = event.getMessage().getOutboundProperty(outboundPropertyName);
                     httpResponseHeaderBuilder.addHeader(outboundPropertyName, outboundPropertyValue);
@@ -97,7 +111,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
             }
         }
 
-        if (!outboundProperties.contains(MuleProperties.CONTENT_TYPE_PROPERTY))
+        if (!outboundProperties.contains(MuleProperties.CONTENT_TYPE_PROPERTY) && event.getMessage().getPayload() != NullPayload.getInstance())
         {
             DataType<?> dataType = event.getMessage().getDataType();
             if (!MimeTypes.ANY.equals(dataType.getMimeType()))
@@ -107,12 +121,24 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
         }
 
         ParameterMap resolvedHeaders = resolveParams(event, HttpParamType.HEADER);
+        if (shouldIncludeConnectionClose(muleContext))
+        {
+            resolvedHeaders.remove(CONNECTION);
+            resolvedHeaders.put(CONNECTION, CLOSE);
+        }
         for (String name : resolvedHeaders.keySet())
         {
             final Collection<String> paramValues = resolvedHeaders.getAll(name);
             for (String value : paramValues)
             {
-                httpResponseHeaderBuilder.addHeader(name, value);
+                if (TRANSFER_ENCODING.equals(name) && !supportsTransferEncoding(event))
+                {
+                    logger.debug("Client HTTP version is lower than 1.1 so the unsupported 'Transfer-Encoding' header has been removed and 'Content-Length' will be sent instead.");
+                }
+                else
+                {
+                    httpResponseHeaderBuilder.addHeader(name, value);
+                }
             }
         }
 
@@ -133,6 +159,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
                 warnNoMultipartContentTypeButMultipartEntity(httpResponseHeaderBuilder.getContentType());
             }
             httpEntity = createMultipartEntity(event, httpResponseHeaderBuilder.getContentType());
+            resolveEncoding(httpResponseHeaderBuilder, existingTransferEncoding, existingContentLength, supportsTransferEncoding(event), (ByteArrayHttpEntity) httpEntity);
         }
         else
         {
@@ -153,41 +180,87 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
                     warnMapPayloadButNoUrlEncodedContentType(httpResponseHeaderBuilder.getContentType());
                 }
                 httpEntity = createUrlEncodedEntity(event, (Map) payload);
-            }
-            else if (payload instanceof InputStream)
-            {
-                if (responseStreaming == ALWAYS || (responseStreaming == AUTO && existingContentLength == null))
+
+                if (responseStreaming == ALWAYS && supportsTransferEncoding(event))
                 {
                     setupChunkedEncoding(httpResponseHeaderBuilder);
-                    httpEntity = new InputStreamHttpEntity((InputStream) payload);
                 }
                 else
                 {
-                    ByteArrayHttpEntity byteArrayHttpEntity = new ByteArrayHttpEntity(IOUtils.toByteArray(((InputStream) payload)));
-                    setupContentLengthEncoding(httpResponseHeaderBuilder, byteArrayHttpEntity.getContent().length);
-                    httpEntity = byteArrayHttpEntity;
-                }
-            }
-            else
-            {
-                try
-                {
-                    ByteArrayHttpEntity byteArrayHttpEntity = new ByteArrayHttpEntity(event.getMessage().getPayloadAsBytes());
-                    if (responseStreaming == ALWAYS || (responseStreaming == AUTO && CHUNKED.equals(existingTransferEncoding)))
+                    if (httpEntity instanceof EmptyHttpEntity)
                     {
-                        setupChunkedEncoding(httpResponseHeaderBuilder);
+                        setupContentLengthEncoding(httpResponseHeaderBuilder, 0);
                     }
                     else
                     {
+                        ByteArrayHttpEntity byteArrayHttpEntity = (ByteArrayHttpEntity) httpEntity;
                         setupContentLengthEncoding(httpResponseHeaderBuilder, byteArrayHttpEntity.getContent().length);
                     }
-                    httpEntity = byteArrayHttpEntity;
-                }
-                catch (Exception e)
-                {
-                    throw new RuntimeException(e);
                 }
             }
+            else {
+                boolean isStream = payload instanceof InputStream;
+                if (isStream || payload instanceof OutputHandler)
+                {
+                    if (responseStreaming == ALWAYS || (responseStreaming == AUTO && existingContentLength == null))
+                    {
+                        if (supportsTransferEncoding(event, httpVersion))
+                        {
+                            setupChunkedEncoding(httpResponseHeaderBuilder);
+                        }
+                        if (isStream) {
+                            httpEntity = new InputStreamHttpEntity((InputStream) payload);
+                        }
+                        else
+                        {
+                            httpEntity = new OutputHandlerHttpEntity((OutputHandler) payload);
+                        }
+                    }
+                    else
+                    {
+                        try
+                        {
+                        ByteArrayHttpEntity byteArrayHttpEntity = new ByteArrayHttpEntity(event.getMessage().getPayload(BYTE_ARRAY_DATA_TYPE));
+                            setupContentLengthEncoding(httpResponseHeaderBuilder, byteArrayHttpEntity.getContent().length);
+                            httpEntity = byteArrayHttpEntity;
+                        }
+                        catch (Exception e)
+                        {
+                            throw new RuntimeException(e);
+                        }
+                    }
+                }
+                else
+                {
+                    try
+                    {
+                        ByteArrayHttpEntity byteArrayHttpEntity = new ByteArrayHttpEntity(event.getMessage().getPayload(BYTE_ARRAY_DATA_TYPE));
+                        resolveEncoding(httpResponseHeaderBuilder, existingTransferEncoding, existingContentLength, supportsTransferEncoding(event), byteArrayHttpEntity);
+                        httpEntity = byteArrayHttpEntity;
+                    }
+                    catch (Exception e)
+                    {
+                        throw new RuntimeException(e);
+                    }
+                }
+            }
+        }
+
+        Integer resolvedStatusCode = resolveStatusCode(event);
+        if (resolvedStatusCode != null)
+        {
+            httpResponseBuilder.setStatusCode(resolvedStatusCode);
+            if (resolvedStatusCode == NO_CONTENT.getStatusCode())
+            {
+                //See GRIZZLY-1885. This could be avoided when that is available
+                httpEntity = new EmptyHttpEntity();
+                httpResponseHeaderBuilder.removeHeader(TRANSFER_ENCODING);
+            }
+        }
+        String resolvedReasonPhrase = resolveReasonPhrase(event, resolvedStatusCode);
+        if (resolvedReasonPhrase != null)
+        {
+            httpResponseBuilder.setReasonPhrase(resolvedReasonPhrase);
         }
 
         Collection<String> headerNames = httpResponseHeaderBuilder.getHeaderNames();
@@ -200,19 +273,55 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
             }
         }
 
-        Integer resolvedStatusCode = resolveStatusCode(event);
-        if (resolvedStatusCode != null)
-        {
-            httpResponseBuilder.setStatusCode(resolvedStatusCode);
-        }
-        if (this.reasonPhrase != null)
-        {
-            httpResponseBuilder.setReasonPhrase(reasonPhraseEvaluator.resolveStringValue(event));
-        }
         httpResponseBuilder.setEntity(httpEntity);
         return httpResponseBuilder.build();
     }
 
+    private static boolean shouldIncludeConnectionClose(MuleContext muleContext)
+    {
+        return muleContext != null && (!muleContext.isStarted() || muleContext.isStopping() || muleContext.isStopped());
+    }
+
+    private boolean supportsTransferEncoding(MuleEvent event)
+    {
+        String httpVersion = event.getMessage().<String> getInboundProperty(HTTP_VERSION_PROPERTY);
+        return !(HTTP_0_9.asString().equals(httpVersion) || HTTP_1_0.asString().equals(httpVersion));
+    }
+
+    private boolean supportsTransferEncoding(MuleEvent event, String httpOriginalVersion)
+    {
+        String httpVersion = event.getMessage().<String> getInboundProperty(HTTP_VERSION_PROPERTY);
+        if (httpVersion == null)
+        {
+            httpVersion = httpOriginalVersion;
+        }
+        return !(HTTP_0_9.asString().equals(httpVersion) || HTTP_1_0.asString().equals(httpVersion));
+    }
+
+    private void resolveEncoding(HttpResponseHeaderBuilder httpResponseHeaderBuilder,
+                                 String existingTransferEncoding,
+                                 String existingContentLength,
+                                 boolean supportsTranferEncoding,
+                                 ByteArrayHttpEntity byteArrayHttpEntity)
+    {
+        if (responseStreaming == ALWAYS || (responseStreaming == AUTO && existingContentLength == null && CHUNKED.equals(existingTransferEncoding)))
+        {
+            if (supportsTranferEncoding)
+            {
+                setupChunkedEncoding(httpResponseHeaderBuilder);
+            }
+        }
+        else
+        {
+            setupContentLengthEncoding(httpResponseHeaderBuilder, byteArrayHttpEntity.getContent().length);
+        }
+    }
+
+    private boolean isNotIgnoredProperty(String outboundPropertyName)
+    {
+        return !outboundPropertyName.startsWith(HTTP_PREFIX) && !outboundPropertyName.equalsIgnoreCase(CONNECTION) && !outboundPropertyName.equalsIgnoreCase(TRANSFER_ENCODING);
+    }
+
     private void setupContentLengthEncoding(HttpResponseHeaderBuilder httpResponseHeaderBuilder, int contentLength)
     {
         if (httpResponseHeaderBuilder.getTransferEncoding() != null)
@@ -220,7 +329,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
             logger.debug("Content-Length encoding is being used so the 'Transfer-Encoding' header has been removed");
             httpResponseHeaderBuilder.removeHeader(TRANSFER_ENCODING);
         }
-        httpResponseHeaderBuilder.addContentLenght(String.valueOf(contentLength));
+        httpResponseHeaderBuilder.setContentLength(String.valueOf(contentLength));
     }
 
     private void setupChunkedEncoding(HttpResponseHeaderBuilder httpResponseHeaderBuilder)
@@ -240,7 +349,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
             return statusCodeEvaluator.resolveIntegerValue(event);
         }
 
-        Object statusCodeOutboundProperty = event.getMessage().getOutboundProperty(HttpConstants.ResponseProperties.HTTP_STATUS_PROPERTY);
+        Object statusCodeOutboundProperty = event.getMessage().getOutboundProperty(HTTP_STATUS_PROPERTY);
         if (statusCodeOutboundProperty != null)
         {
             return NumberUtils.toInt(statusCodeOutboundProperty);
@@ -249,6 +358,26 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
         return null;
     }
 
+    private String resolveReasonPhrase(MuleEvent event, Integer resolvedStatusCode)
+    {
+        if (reasonPhrase != null)
+        {
+            return reasonPhraseEvaluator.resolveStringValue(event);
+        }
+
+        Object reasonPhraseOutboundProperty = event.getMessage().getOutboundProperty(HTTP_REASON_PROPERTY);
+        if (reasonPhraseOutboundProperty != null)
+        {
+            return reasonPhraseOutboundProperty.toString();
+        }
+        else if(resolvedStatusCode != null)
+        {
+            return getReasonPhraseForStatusCode(resolvedStatusCode);
+        }
+
+        return null;
+    }
+
     private String createMultipartFormDataContentType()
     {
         return String.format("%s; boundary=%s", HttpHeaders.Values.MULTIPART_FORM_DATA, UUID.getUUID());
@@ -276,19 +405,19 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
 
     private void warnMapPayloadButNoUrlEncodedContentType(String contentType)
     {
-        if (!mapPayloadButNoUrlEncodedContentyTypeWarned)
+        if (!mapPayloadButNoUrlEncodedContentTypeWarned)
         {
-            logger.warn(String.format("Payload is a Map which will be used to generate an url encoded http body but Contenty-Type specified is %s and not %s", contentType, HttpHeaders.Values.APPLICATION_X_WWW_FORM_URLENCODED));
-            mapPayloadButNoUrlEncodedContentyTypeWarned = true;
+            logger.warn(String.format("Payload is a Map which will be used to generate an url encoded http body but Content-Type specified is %s and not %s", contentType, HttpHeaders.Values.APPLICATION_X_WWW_FORM_URLENCODED));
+            mapPayloadButNoUrlEncodedContentTypeWarned = true;
         }
     }
 
     private void warnNoMultipartContentTypeButMultipartEntity(String contentType)
     {
-        if (!multipartEntityWithNoMultipartContentyTypeWarned)
+        if (!multipartEntityWithNoMultipartContentTypeWarned)
         {
             logger.warn(String.format("Sending http response with Content-Type %s but the message has attachment and a multipart entity is generated", contentType));
-            multipartEntityWithNoMultipartContentyTypeWarned = true;
+            multipartEntityWithNoMultipartContentTypeWarned = true;
         }
     }
 
@@ -298,7 +427,7 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
         {
             logger.debug("Message contains outbound attachments. Ignoring payload and trying to generate multipart response");
         }
-        final HashMap<String, DataHandler> parts = new HashMap<>();
+        final HashMap<String, DataHandler> parts = new LinkedHashMap<>();
         for (String outboundAttachmentName : event.getMessage().getOutboundAttachmentNames())
         {
             parts.put(outboundAttachmentName, event.getMessage().getOutboundAttachment(outboundAttachmentName));
@@ -354,4 +483,4 @@ public class HttpResponseBuilder extends HttpMessageBuilder implements Initialis
     {
         this.muleContext = context;
     }
-}
\ No newline at end of file
+}
