@@ -1,13 +1,9 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.module.pgp.i18n;
 
 import org.mule.config.i18n.Message;
@@ -24,11 +20,6 @@ public class PGPMessages extends MessageFactory
         return factory.createMessage(BUNDLE_PATH, 1, userId);
     }
 
-    public static Message noSignedMessageFound()
-    {
-        return factory.createMessage(BUNDLE_PATH, 2);
-    }
-
     public static Message invalidSignature()
     {
         return factory.createMessage(BUNDLE_PATH, 3);
@@ -48,12 +39,36 @@ public class PGPMessages extends MessageFactory
     {
         return factory.createMessage(BUNDLE_PATH, 6);
     }
-    
-    public static Message noSecretKeyFoundButAvailable(String availableKeys)
+
+    public static Message noPublicKeyForPrincipal(String principalId)
     {
-        return factory.createMessage(BUNDLE_PATH, 7, availableKeys);
+        return factory.createMessage(BUNDLE_PATH, 8, principalId);
     }
 
+    public static Message noFileKeyFound(String path)
+    {
+        return factory.createMessage(BUNDLE_PATH, 9, path);
+    }
+
+    public static Message noSecretKeyDefined()
+    {
+        return factory.createMessage(BUNDLE_PATH, 10);
+    }
+
+    public static Message noSecretPassPhrase()
+    {
+        return factory.createMessage(BUNDLE_PATH, 11);
+    }
+
+    public static Message noKeyIdFound(String id)
+    {
+        return factory.createMessage(BUNDLE_PATH, 12, id);
+    }
+
+    public static Message noPublicKeyDefined()
+    {
+        return factory.createMessage(BUNDLE_PATH, 13);
+    }
 }
 
 
