@@ -61,6 +61,7 @@ import org.springframework.context.ApplicationListener;
 import org.springframework.context.event.ApplicationEventMulticaster;
 import org.springframework.context.event.ContextClosedEvent;
 import org.springframework.context.event.ContextRefreshedEvent;
+import org.springframework.core.ResolvableType;
 
 /**
  * <code>MuleEventMulticaster</code> is an implementation of a Spring
@@ -108,7 +109,10 @@ import org.springframework.context.event.ContextRefreshedEvent;
  * @see MuleEventListener
  * @see MuleSubscriptionEventListener
  * @see ApplicationEventMulticaster
+ *
+ * <b>Deprecated from 3.6.0.</b>
  */
+@Deprecated
 public class MuleEventMulticaster
     implements ApplicationEventMulticaster, ApplicationContextAware, MuleContextAware, Callable, Initialisable
 {
@@ -409,6 +413,17 @@ public class MuleEventMulticaster
         }
     }
 
+    /***
+     * Implemented only for compliance with Spring 4.2+
+     * Please use the {@link #multicastEvent(ApplicationEvent) multicastEvent} method.
+     * @param event
+     * @param eventType
+     */
+    @Override
+    public void multicastEvent(ApplicationEvent event, ResolvableType eventType) {
+        multicastEvent(event);
+    }
+
     /**
      * Matches a subscription to the current event endpointUri
      *
