@@ -1,58 +1,75 @@
 /*
- * $Id: AbstractXPathExpressionEvaluator.java 22645 2011-08-10 20:52:36Z evangelinamrm $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
 package org.mule.module.xml.expression;
 
+import static org.mule.api.config.MuleProperties.SYSTEM_PROPERTY_PREFIX;
+
 import org.mule.api.MuleContext;
 import org.mule.api.MuleMessage;
 import org.mule.api.MuleRuntimeException;
 import org.mule.api.context.MuleContextAware;
 import org.mule.api.context.notification.MuleContextNotificationListener;
-import org.mule.api.expression.ExpressionEvaluator;
 import org.mule.api.expression.ExpressionRuntimeException;
 import org.mule.api.lifecycle.Disposable;
 import org.mule.api.lifecycle.Initialisable;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.registry.RegistrationException;
+import org.mule.api.transformer.TransformerException;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.context.notification.MuleContextNotification;
+import org.mule.expression.AbstractExpressionEvaluator;
+import org.mule.module.xml.el.XPath3Function;
 import org.mule.module.xml.i18n.XmlMessages;
 import org.mule.module.xml.util.NamespaceManager;
+import org.mule.transformer.types.DataTypeFactory;
+import org.mule.util.OneTimeWarning;
 
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.WeakHashMap;
 
 import org.dom4j.Document;
 import org.jaxen.JaxenException;
 import org.jaxen.XPath;
-import org.mule.transformer.types.DataTypeFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.w3c.dom.Node;
+
+import com.google.common.cache.CacheBuilder;
 
 /**
  * Provides a base class for XPath property extractors. The XPath engine used is jaxen (http://jaxen.org) which supports
  * XPath queries on other object models such as JavaBeans as well as Xml
+ *
+ * @deprecated this base class is deprecated. Use {@link XPath3Function} instead
  */
-public abstract class AbstractXPathExpressionEvaluator implements ExpressionEvaluator, Initialisable, Disposable, MuleContextAware
+@Deprecated
+public abstract class AbstractXPathExpressionEvaluator extends AbstractExpressionEvaluator implements Initialisable, Disposable, MuleContextAware
 {
-    private Map<String, XPath> cache = new WeakHashMap<String, XPath>(8);
+    private static final int MAX_CACHE_SIZE = Integer.getInteger(SYSTEM_PROPERTY_PREFIX + "xml.xpath.cacheSize", 256);
 
+    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractXPathExpressionEvaluator.class);
+
+    private final Map<String, XPath> cache = CacheBuilder.newBuilder().initialCapacity(8).maximumSize(MAX_CACHE_SIZE)
+            .<String, XPath>build().asMap();
+    
     private MuleContext muleContext;
     private NamespaceManager namespaceManager;
+    private OneTimeWarning deprecationWarning = new OneTimeWarning(LOGGER, getDeprecationMessage());
 
+    protected abstract String getDeprecationMessage();
+
+    @Override
     public void setMuleContext(MuleContext context)
     {
         this.muleContext = context;
-
     }
 
+    @Override
     public void initialise() throws InitialisationException
     {
         try
@@ -67,6 +84,7 @@ public abstract class AbstractXPathExpressionEvaluator implements ExpressionEval
              */
             this.muleContext.registerListener(new MuleContextNotificationListener<MuleContextNotification>(){
 
+                @Override
                 public void onNotification(MuleContextNotification notification)
                 {
                     // CONTEXT_INITIALIZED fires too soon, before registry is inited, thus using this one
@@ -103,24 +121,25 @@ public abstract class AbstractXPathExpressionEvaluator implements ExpressionEval
     }
 
     /** {@inheritDoc} */
+    @Override
     public Object evaluate(String expression, MuleMessage message)
     {
         try
         {
-            Object payload = message.getPayload();
-            //we need to convert to a Dom if its an XML string
-            if(payload instanceof String)
-            {
-                payload = message.getPayload(DataTypeFactory.create(Document.class));
-            }
+            deprecationWarning.warn();
+            Object payload = getPayloadForXPath(message);
 
+            List<?> result;
+
+            /*  XPath context state is not thread safe so synchronization must be enforced when adding a new namespace and
+                on evaluation when the context is read.
+             */
             XPath xpath = getXPath(expression, payload);
-            if(namespaceManager!=null)
+            synchronized (xpath)
             {
-                addNamespaces(namespaceManager, xpath);
+                result = xpath.selectNodes(payload);
             }
 
-            List<?> result = xpath.selectNodes(payload);
             result = extractResultsFromNodes(result);
             if(result.size()==1)
             {
@@ -141,42 +160,61 @@ public abstract class AbstractXPathExpressionEvaluator implements ExpressionEval
         }
     }
 
+    protected Object getPayloadForXPath(MuleMessage message) throws TransformerException
+    {
+        Object payload = message.getPayload();
+        //we need to convert to a Dom if its an XML string
+        if(!(payload instanceof Node || payload instanceof org.dom4j.Node))
+        {
+            payload = message.getPayload(DataTypeFactory.create(Document.class));
+        }
+        return payload;
+    }
+
     protected void addNamespaces(NamespaceManager manager, XPath xpath)
     {
-        for (Iterator<?> iterator = manager.getNamespaces().entrySet().iterator(); iterator.hasNext();)
+        for (Map.Entry<String, String> entry : manager.getNamespaces().entrySet())
         {
-            Map.Entry<?, ?> entry = (Map.Entry<?, ?>)iterator.next();
             try
             {
-                synchronized (xpath)
-                {
-                    xpath.addNamespace(entry.getKey().toString(), entry.getValue().toString());
-                }
+                xpath.addNamespace(entry.getKey(), entry.getValue());
             }
             catch (JaxenException e)
             {
-                throw new ExpressionRuntimeException(XmlMessages.failedToRegisterNamespace(entry.getKey().toString(), entry.getValue().toString()));
+                throw new ExpressionRuntimeException(XmlMessages.failedToRegisterNamespace(entry.getKey(), entry.getValue()));
             }
         }
     }
 
-    /** {@inheritDoc} */
-    public final void setName(String name)
-    {
-        throw new UnsupportedOperationException("setName");
-    }
-
+    /*
+        The cache is not thread safe, more than one instance of the same xpath can be created, it wouldn't be an
+        issue in this case since one will eventually be selected for GC
+        A ReadWriteLock could be used to guard the cache but it would add unnecessary complexity
+     */
     protected XPath getXPath(String expression, Object object) throws JaxenException
     {
-        XPath xpath = cache.get(expression + getClass().getName());
+        String xPathCacheKey = expression + getXPathClassName(object);
+        XPath xpath = cache.get(xPathCacheKey);
         if(xpath==null)
         {
             xpath = createXPath(expression, object);
-            cache.put(expression + getClass().getName(), xpath);
+            synchronized (xpath)
+            {
+                if(namespaceManager!=null)
+                {
+                    addNamespaces(namespaceManager, xpath);
+                }
+            }
+            cache.put(xPathCacheKey, xpath);
         }
         return xpath;
     }
 
+    protected String getXPathClassName(Object object)
+    {
+        return getClass().getName();
+    }
+
     protected abstract XPath createXPath(String expression, Object object) throws JaxenException;
 
     protected List<?> extractResultsFromNodes(List<?> results)
@@ -198,6 +236,7 @@ public abstract class AbstractXPathExpressionEvaluator implements ExpressionEval
      * exception is thrown it should just be logged and processing should continue.
      * This method should not throw Runtime exceptions.
      */
+    @Override
     public void dispose()
     {
         cache.clear();
