@@ -1,13 +1,9 @@
 /*
- * $Id: MuleRegistryHelper.java 24423 2012-05-28 03:15:12Z pablo.kraan $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.registry;
 
 import org.mule.api.MuleContext;
@@ -25,14 +21,18 @@ import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.lifecycle.LifecycleException;
 import org.mule.api.model.Model;
 import org.mule.api.registry.AbstractServiceDescriptor;
+import org.mule.api.registry.LifecycleRegistry;
 import org.mule.api.registry.MuleRegistry;
 import org.mule.api.registry.RegistrationException;
+import org.mule.api.registry.Registry;
+import org.mule.api.registry.RegistryProvider;
 import org.mule.api.registry.ResolverException;
 import org.mule.api.registry.ServiceDescriptor;
 import org.mule.api.registry.ServiceDescriptorFactory;
 import org.mule.api.registry.ServiceException;
 import org.mule.api.registry.ServiceType;
 import org.mule.api.registry.TransformerResolver;
+import org.mule.api.schedule.Scheduler;
 import org.mule.api.service.Service;
 import org.mule.api.transformer.Converter;
 import org.mule.api.transformer.DataType;
@@ -41,19 +41,29 @@ import org.mule.api.transformer.TransformerException;
 import org.mule.api.transport.Connector;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.transformer.types.SimpleDataType;
+import org.mule.util.Predicate;
 import org.mule.util.SpiUtils;
 import org.mule.util.StringUtils;
 import org.mule.util.UUID;
 
+import com.google.common.collect.ImmutableList;
+
+import static java.util.Collections.synchronizedMap;
+import static org.mule.transformer.TransformerUtils.getConverterKey;
+
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -62,7 +72,7 @@ import org.apache.commons.logging.LogFactory;
  * Adds lookup/register/unregister methods for Mule-specific entities to the standard
  * Registry interface.
  */
-public class MuleRegistryHelper implements MuleRegistry
+public class MuleRegistryHelper implements MuleRegistry, RegistryProvider
 {
     protected transient Log logger = LogFactory.getLog(MuleRegistryHelper.class);
 
@@ -79,6 +89,20 @@ public class MuleRegistryHelper implements MuleRegistry
 
     private MuleContext muleContext;
 
+    private final ReadWriteLock transformerResolversLock = new ReentrantReadWriteLock();
+
+    /**
+     * Transformer transformerResolvers are registered on context start, then they are not unregistered.
+     */
+    private List<TransformerResolver> transformerResolvers = new ArrayList<TransformerResolver>();
+
+    private final ReadWriteLock transformersLock = new ReentrantReadWriteLock();
+
+    /**
+     * Transformers are registered on context start, then they are usually not unregistered
+     */
+    private Map<String, Transformer> transformers = synchronizedMap(new LinkedHashMap<String, Transformer>());
+
     public MuleRegistryHelper(DefaultRegistryBroker registry, MuleContext muleContext)
     {
         this.registry = registry;
@@ -107,11 +131,11 @@ public class MuleRegistryHelper implements MuleRegistry
 
     public void fireLifecycle(String phase) throws LifecycleException
     {
-        if(Initialisable.PHASE_NAME.equals(phase))
+        if (Initialisable.PHASE_NAME.equals(phase))
         {
             registry.initialise();
         }
-        else if(Disposable.PHASE_NAME.equals(phase))
+        else if (Disposable.PHASE_NAME.equals(phase))
         {
             registry.dispose();
         }
@@ -129,37 +153,6 @@ public class MuleRegistryHelper implements MuleRegistry
         return (Connector) registry.lookupObject(name);
     }
 
-    /**
-     * Removed this method from {@link Registry} API as it should only be used
-     * internally and may confuse users. The {@link EndpointFactory} should be used
-     * for creating endpoints.<br/><br/> Looks up an returns endpoints registered in the
-     * registry by their identifier (currently endpoint name)<br/><br/ <b>NOTE:
-     * This method does not create new endpoint instances, but rather returns
-     * existing endpoint instances that have been registered. This lookup method
-     * should be avoided and the intelligent, role specific endpoint lookup methods
-     * should be used instead.<br/><br/>
-     *
-     * @param name the idendtifer/name used to register endpoint in registry
-     * @return foo
-     */
-    /*public ImmutableEndpoint lookupEndpoint(String name)
-    {
-        Object obj = registry.lookupObject(name);
-        if (obj instanceof ImmutableEndpoint)
-        {
-            return (ImmutableEndpoint) obj;
-        }
-        else
-        {
-            logger.debug("No endpoint with the name: "
-                    + name
-                    + "found.  If "
-                    + name
-                    + " is a global endpoint you should use the EndpointFactory to create endpoint instances from global endpoints.");
-            return null;
-        }
-    }*/
-
     /**
      * {@inheritDoc}
      */
@@ -168,12 +161,18 @@ public class MuleRegistryHelper implements MuleRegistry
         Object o = registry.lookupObject(name);
         if (o instanceof EndpointBuilder)
         {
-            logger.debug("Global endpoint EndpointBuilder for name: " + name + " found");
+            if (logger.isDebugEnabled())
+            {
+                logger.debug("Global endpoint EndpointBuilder for name: " + name + " found");
+            }
             return (EndpointBuilder) o;
         }
         else
         {
-            logger.debug("No endpoint builder with the name: " + name + " found.");
+            if (logger.isDebugEnabled())
+            {
+                logger.debug("No endpoint builder with the name: " + name + " found.");
+            }
             return null;
         }
     }
@@ -198,8 +197,8 @@ public class MuleRegistryHelper implements MuleRegistry
      * {@inheritDoc}
      *
      * @deprecated use {@link #lookupTransformer(org.mule.api.transformer.DataType, org.mule.api.transformer.DataType)} instead.  This
-     * method should only be used internally to discover transformers, typically a user does not need ot do this
-     * directly
+     *             method should only be used internally to discover transformers, typically a user does not need ot do this
+     *             directly
      */
     @Deprecated
     public Transformer lookupTransformer(Class inputType, Class outputType) throws TransformerException
@@ -211,8 +210,8 @@ public class MuleRegistryHelper implements MuleRegistry
      * {@inheritDoc}
      *
      * @deprecated use {@link #lookupTransformer(org.mule.api.transformer.DataType, org.mule.api.transformer.DataType)} instead.  This
-     * method should only be used internally to discover transformers, typically a user does not need ot do this
-     * directly
+     *             method should only be used internally to discover transformers, typically a user does not need ot do this
+     *             directly
      */
     @Deprecated
     public List<Transformer> lookupTransformers(Class input, Class output)
@@ -237,7 +236,7 @@ public class MuleRegistryHelper implements MuleRegistry
         if (trans != null)
         {
             Transformer concurrentlyAddedTransformer = (Transformer) exactTransformerCache.putIfAbsent(
-                dataTypePairHash, trans);
+                    dataTypePairHash, trans);
             if (concurrentlyAddedTransformer != null)
             {
                 return concurrentlyAddedTransformer;
@@ -255,24 +254,32 @@ public class MuleRegistryHelper implements MuleRegistry
 
     protected Transformer resolveTransformer(DataType source, DataType result) throws TransformerException
     {
-        List<TransformerResolver> resolvers = (List<TransformerResolver>) lookupObjects(TransformerResolver.class);
-        Collections.sort(resolvers, new TransformerResolverComparator());
+        Lock readLock = transformerResolversLock.readLock();
+        readLock.lock();
 
-        for (TransformerResolver resolver : resolvers)
+        try
         {
-            try
+            for (TransformerResolver resolver : transformerResolvers)
             {
-                Transformer trans = resolver.resolve(source, result);
-                if (trans != null)
+                try
                 {
-                    return trans;
+                    Transformer trans = resolver.resolve(source, result);
+                    if (trans != null)
+                    {
+                        return trans;
+                    }
+                }
+                catch (ResolverException e)
+                {
+                    throw new TransformerException(CoreMessages.noTransformerFoundForMessage(source, result), e);
                 }
             }
-            catch (ResolverException e)
-            {
-                throw new TransformerException(CoreMessages.noTransformerFoundForMessage(source, result), e);
-            }
         }
+        finally
+        {
+            readLock.unlock();
+        }
+
         return null;
     }
 
@@ -290,25 +297,33 @@ public class MuleRegistryHelper implements MuleRegistry
         }
 
         results = new ArrayList<Transformer>(2);
-        Collection<Transformer> transformers = registry.lookupObjects(Transformer.class);
-        for (Transformer t : transformers)
+
+        Lock readLock = transformersLock.readLock();
+        readLock.lock();
+        try
         {
-            // The transformer must have the DiscoveryTransformer interface if we are
-            // going to
-            // find it here
-            if (!(t instanceof Converter))
+            for (Transformer transformer : transformers.values())
             {
-                continue;
-            }
-            DataType dt = t.getReturnDataType();
-            if (result.isCompatibleWith(dt) && t.isSourceDataTypeSupported(source))
-            {
-                results.add(t);
+                // The transformer must have the DiscoveryTransformer interface if we are
+                // going to find it here
+                if (!(transformer instanceof Converter))
+                {
+                    continue;
+                }
+                DataType returnDataType = transformer.getReturnDataType();
+                if (result.isCompatibleWith(returnDataType) && transformer.isSourceDataTypeSupported(source))
+                {
+                    results.add(transformer);
+                }
             }
         }
+        finally
+        {
+            readLock.unlock();
+        }
 
         List<Transformer> concurrentlyAddedTransformers = (List<Transformer>) transformerListCache.putIfAbsent(
-            dataTypePairHash, results);
+                dataTypePairHash, results);
         if (concurrentlyAddedTransformers != null)
         {
             return concurrentlyAddedTransformers;
@@ -322,6 +337,7 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Deprecated
     public Model lookupModel(String name)
     {
         return (Model) registry.lookupObject(name);
@@ -330,6 +346,7 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Deprecated
     public Model lookupSystemModel()
     {
         return lookupModel(MuleProperties.OBJECT_SYSTEM_MODEL);
@@ -338,6 +355,7 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Deprecated
     public Collection<Model> getModels()
     {
         return registry.lookupObjects(Model.class);
@@ -440,21 +458,48 @@ public class MuleRegistryHelper implements MuleRegistry
      */
     public final void registerTransformer(Transformer transformer) throws MuleException
     {
-        registry.registerObject(getName(transformer), transformer, Transformer.class);
-        notifyTransformerResolvers(transformer, TransformerResolver.RegistryAction.ADDED);
+        registerObject(getName(transformer), transformer, Transformer.class);
     }
 
     public void notifyTransformerResolvers(Transformer t, TransformerResolver.RegistryAction action)
     {
         if (t instanceof Converter)
         {
-            Collection<TransformerResolver> resolvers = lookupObjects(TransformerResolver.class);
-            for (TransformerResolver resolver : resolvers)
+            Lock transformerResolversReadLock = transformerResolversLock.readLock();
+            transformerResolversReadLock.lock();
+            try
             {
-                resolver.transformerChange(t, action);
+
+                for (TransformerResolver resolver : transformerResolvers)
+                {
+                    resolver.transformerChange(t, action);
+                }
             }
+            finally
+            {
+                transformerResolversReadLock.unlock();
+            }
+
             transformerListCache.clear();
             exactTransformerCache.clear();
+
+            Lock transformersWriteLock = transformersLock.writeLock();
+            transformersWriteLock.lock();
+            try
+            {
+                if (action == TransformerResolver.RegistryAction.ADDED)
+                {
+                    transformers.put(getConverterKey(t), t);
+                }
+                else
+                {
+                    transformers.remove(getConverterKey(t));
+                }
+            }
+            finally
+            {
+                transformersWriteLock.unlock();
+            }
         }
     }
 
@@ -466,10 +511,10 @@ public class MuleRegistryHelper implements MuleRegistry
         String key = new AbstractServiceDescriptor.Key(name, overrides).getKey();
         // TODO If we want these descriptors loaded form Spring we need to change the key mechanism
         // and the scope, and then deal with circular reference issues.
-        ServiceDescriptor sd = (ServiceDescriptor) registry.lookupObject(key);
 
         synchronized (this)
         {
+            ServiceDescriptor sd = registry.lookupObject(key);
             if (sd == null)
             {
                 sd = createServiceDescriptor(type, name, overrides);
@@ -482,8 +527,8 @@ public class MuleRegistryHelper implements MuleRegistry
                     throw new ServiceException(e.getI18nMessage(), e);
                 }
             }
+            return sd;
         }
-        return sd;
     }
 
     protected ServiceDescriptor createServiceDescriptor(ServiceType type, String name, Properties overrides) throws ServiceException
@@ -540,6 +585,7 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Deprecated
     public void registerModel(Model model) throws MuleException
     {
         registry.registerObject(getName(model), model, Model.class);
@@ -585,6 +631,34 @@ public class MuleRegistryHelper implements MuleRegistry
         registry.unregisterObject(agentName, Agent.class);
     }
 
+    @Override
+    public void registerScheduler(Scheduler scheduler) throws MuleException
+    {
+        registry.registerObject(scheduler.getName(), scheduler);
+    }
+
+    @Override
+    public void unregisterScheduler(Scheduler scheduler) throws MuleException
+    {
+        registry.unregisterObject(scheduler.getName(), scheduler);
+    }
+
+    @Override
+    public Collection<Scheduler> lookupScheduler(Predicate<String> schedulerNamePredicate)
+    {
+        Collection<Scheduler> schedulers = new ArrayList<Scheduler>();
+        Map<String, Scheduler> registeredSchedulers = lookupByType(Scheduler.class);
+        for (Scheduler registeredScheduler : registeredSchedulers.values())
+        {
+            if (schedulerNamePredicate.evaluate(registeredScheduler.getName()))
+            {
+                schedulers.add(registeredScheduler);
+            }
+        }
+
+        return schedulers;
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -604,6 +678,7 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Deprecated
     public void unregisterModel(String modelName) throws MuleException
     {
         registry.unregisterObject(modelName, Model.class);
@@ -633,30 +708,44 @@ public class MuleRegistryHelper implements MuleRegistry
     /**
      * {@inheritDoc}
      */
+    @Override
     public Object applyProcessors(Object object) throws MuleException
     {
-        return registry.getTransientRegistry().applyProcessors(object, null);
+        return muleContext.getInjector().inject(object);
     }
 
     /**
      * {@inheritDoc}
      */
+    @Override
+    @Deprecated
     public Object applyProcessors(Object object, int flags) throws MuleException
     {
-        return registry.getTransientRegistry().applyProcessors(object, flags);
+        return applyProcessors(object);
     }
 
     /**
      * {@inheritDoc}
      */
+    @Override
     public Object applyLifecycle(Object object) throws MuleException
     {
-        return registry.getTransientRegistry().applyLifecycle(object);
+        return applyLifecycle(object, null);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
     public Object applyLifecycle(Object object, String phase) throws MuleException
     {
-        return registry.getTransientRegistry().applyLifecycle(object, phase);
+        LifecycleRegistry lifecycleRegistry = registry.getLifecycleRegistry();
+        if (lifecycleRegistry != null)
+        {
+            return lifecycleRegistry.applyLifecycle(object, phase);
+        }
+
+        return object;
     }
 
     ////////////////////////////////////////////////////////////////////////////
@@ -671,12 +760,24 @@ public class MuleRegistryHelper implements MuleRegistry
         return registry.lookupObject(type);
     }
 
+    /**
+     * {@inheritDoc}
+     */
     @SuppressWarnings("unchecked")
     public <T> T lookupObject(String key)
     {
         return (T) registry.lookupObject(key);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public <T> T lookupObject(String key, boolean applyLifecycle)
+    {
+        return (T) registry.lookupObject(key, applyLifecycle);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -685,6 +786,12 @@ public class MuleRegistryHelper implements MuleRegistry
         return registry.lookupObjects(type);
     }
 
+    @Override
+    public <T> Collection<T> lookupLocalObjects(Class<T> type)
+    {
+        return registry.lookupLocalObjects(type);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -696,7 +803,7 @@ public class MuleRegistryHelper implements MuleRegistry
     @SuppressWarnings("unchecked")
     public <T> T get(String key)
     {
-        return (T)registry.get(key);
+        return (T) registry.get(key);
     }
 
     public <T> Map<String, T> lookupByType(Class<T> type)
@@ -710,6 +817,21 @@ public class MuleRegistryHelper implements MuleRegistry
     public void registerObject(String key, Object value, Object metadata) throws RegistrationException
     {
         registry.registerObject(key, value, metadata);
+
+        postObjectRegistrationActions(value);
+    }
+
+    public void postObjectRegistrationActions(Object value)
+    {
+        if (value instanceof TransformerResolver)
+        {
+            registerTransformerResolver((TransformerResolver) value);
+        }
+
+        if (value instanceof Converter)
+        {
+            notifyTransformerResolvers((Converter) value,  TransformerResolver.RegistryAction.ADDED);
+        }
     }
 
     /**
@@ -718,6 +840,23 @@ public class MuleRegistryHelper implements MuleRegistry
     public void registerObject(String key, Object value) throws RegistrationException
     {
         registry.registerObject(key, value);
+
+        postObjectRegistrationActions(value);
+    }
+
+    public void registerTransformerResolver(TransformerResolver value)
+    {
+        Lock lock = transformerResolversLock.writeLock();
+        lock.lock();
+        try
+        {
+            transformerResolvers.add(value);
+            Collections.sort(transformerResolvers, new TransformerResolverComparator());
+        }
+        finally
+        {
+            lock.unlock();
+        }
     }
 
     /**
@@ -726,28 +865,40 @@ public class MuleRegistryHelper implements MuleRegistry
     public void registerObjects(Map objects) throws RegistrationException
     {
         registry.registerObjects(objects);
+
+        for (Object value : objects.values())
+        {
+            postObjectRegistrationActions(value);
+        }
     }
 
     /**
      * {@inheritDoc}
      */
-    public void unregisterObject(String key, Object metadata) throws RegistrationException
+    public Object unregisterObject(String key, Object metadata) throws RegistrationException
     {
-        registry.unregisterObject(key, metadata);
+        return registry.unregisterObject(key, metadata);
     }
 
     /**
      * {@inheritDoc}
      */
-    public void unregisterObject(String key) throws RegistrationException
+    public Object unregisterObject(String key) throws RegistrationException
     {
-        registry.unregisterObject(key);
+        return registry.unregisterObject(key);
+    }
+
+    @Override
+    public Collection<Registry> getRegistries()
+    {
+        return ImmutableList.copyOf(registry.getRegistries());
     }
 
     /**
      * Returns the name for the object passed in.  If the object implements {@link org.mule.api.NameableObject}, then
      * {@link org.mule.api.NameableObject#getName()} will be returned, otherwise a name is generated using the class name
      * and a generated UUID.
+     *
      * @param obj the object to inspect
      * @return the name for this object
      */
@@ -805,6 +956,7 @@ public class MuleRegistryHelper implements MuleRegistry
 
     private class TransformerResolverComparator implements Comparator<TransformerResolver>
     {
+
         public int compare(TransformerResolver transformerResolver, TransformerResolver transformerResolver1)
         {
             if (transformerResolver.getClass().equals(TypeBasedTransformerResolver.class))
@@ -821,4 +973,3 @@ public class MuleRegistryHelper implements MuleRegistry
     }
 }
 
-
