@@ -1,13 +1,9 @@
 /*
- * $Id: AbstractSingleResourceTransaction.java 23961 2012-03-07 16:48:42Z pablo.lagreca $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.transaction;
 
 import org.mule.api.MuleContext;
@@ -170,8 +166,7 @@ public abstract class AbstractSingleResourceTransaction extends AbstractTransact
         return new StringBuilder().append(getClass().getName())
                 .append('@').append(id)
                 .append("[status=").append(statusName)
-                .append(", key=").append(key)
-                .append(", resource=").append(resource)
+                .append(", key=").append(keyToString())
                 .append("]").toString();
     }
 
@@ -190,4 +185,9 @@ public abstract class AbstractSingleResourceTransaction extends AbstractTransact
     {
         throw new MuleRuntimeException(CoreMessages.createStaticMessage("Transaction type: " + this.getClass().getName() + " doesn't support supports(..) method"));
     }
+    
+    protected Object keyToString()
+    {
+        return key;
+    }
 }
