@@ -1,15 +1,14 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.transformer.simple;
 
+import static java.lang.Boolean.parseBoolean;
+import static java.lang.System.getProperty;
+import static org.mule.api.config.MuleProperties.MULE_ENABLE_BYTE_ARRAY_TO_INPUT_STREAM;
 import org.mule.RequestContext;
 import org.mule.api.transformer.TransformerException;
 import org.mule.api.transport.OutputHandler;
@@ -26,9 +25,14 @@ import java.io.ByteArrayOutputStream;
  */
 public class ObjectToInputStream extends SerializableToByteArray
 {
+    private boolean acceptsByteArray = parseBoolean(getProperty(MULE_ENABLE_BYTE_ARRAY_TO_INPUT_STREAM, "false"));
 
     public ObjectToInputStream()
     {
+        if (acceptsByteArray)
+        {
+            this.registerSourceType(DataTypeFactory.BYTE_ARRAY);
+        }
         this.registerSourceType(DataTypeFactory.STRING);
         this.registerSourceType(DataTypeFactory.create(OutputHandler.class));
         setReturnDataType(DataTypeFactory.INPUT_STREAM);
