@@ -6,12 +6,16 @@
  */
 package org.mule.transport.file;
 
+import static java.lang.Boolean.getBoolean;
+import static org.mule.api.config.MuleProperties.SYSTEM_PROPERTY_PREFIX;
+import static org.mule.transport.file.FileConnector.PROPERTY_FILE_AGE;
 import org.mule.DefaultMuleMessage;
 import org.mule.api.DefaultMuleException;
 import org.mule.api.MessagingException;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleMessage;
+import org.mule.api.MuleRuntimeException;
 import org.mule.api.config.MuleProperties;
 import org.mule.api.construct.FlowConstruct;
 import org.mule.api.endpoint.InboundEndpoint;
@@ -25,6 +29,7 @@ import org.mule.api.store.ObjectStoreException;
 import org.mule.api.store.ObjectStoreManager;
 import org.mule.api.transport.Connector;
 import org.mule.api.transport.PropertyScope;
+import org.mule.config.i18n.CoreMessages;
 import org.mule.construct.Flow;
 import org.mule.processor.strategy.SynchronousProcessingStrategy;
 import org.mule.transport.AbstractPollingMessageReceiver;
@@ -57,12 +62,15 @@ import org.apache.commons.collections.comparators.ReverseComparator;
 
 public class FileMessageReceiver extends AbstractPollingMessageReceiver
 {
+
+    private static final String RECENTLY_PROCESSED_SUFFIX = "_recently_processed";
     public static final String COMPARATOR_CLASS_NAME_PROPERTY = "comparator";
     public static final String COMPARATOR_REVERSE_ORDER_PROPERTY = "reverseOrder";
     public static final String MULE_TRANSPORT_FILE_SINGLEPOLLINSTANCE = "mule.transport.file.singlepollinstance";
+    public static final String MULE_NOT_PROCESSING_RECENTLY_FILES_PERIOD = "mule.transport.file.not.recently.files.processing.period";
+    public static final String IGNORE_EMPTY_FILES_PROPERTY = SYSTEM_PROPERTY_PREFIX + "transport.file.ignoreEmptyFiles";
 
-    private static final List<File> NO_FILES = new ArrayList<File>();
-
+    private final Boolean ignoreEmptyFiles = getBoolean(IGNORE_EMPTY_FILES_PROPERTY);
     private FileConnector fileConnector = null;
     private String readDir = null;
     private String moveDir = null;
@@ -76,7 +84,9 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
     private boolean forceSync;
     private LockFactory lockFactory;
     private boolean poolOnPrimaryInstanceOnly;
+    private Integer notProcessingRecentlyFilesPeriod;
     private ObjectStore<String> filesBeingProcessingObjectStore;
+    private ObjectStore<String> filesRecentlyProcessed;
 
     public FileMessageReceiver(Connector connector,
                                FlowConstruct flowConstruct,
@@ -138,11 +148,13 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         boolean synchronousProcessing = false;
         if (getFlowConstruct() instanceof Flow)
         {
-            synchronousProcessing = ((Flow)getFlowConstruct()).getProcessingStrategy() instanceof SynchronousProcessingStrategy;
+            synchronousProcessing = ((Flow) getFlowConstruct()).getProcessingStrategy() instanceof SynchronousProcessingStrategy;
         }
-        this.poolOnPrimaryInstanceOnly = Boolean.valueOf(System.getProperty(MULE_TRANSPORT_FILE_SINGLEPOLLINSTANCE,"false")) || !synchronousProcessing;
+        this.poolOnPrimaryInstanceOnly = Boolean.valueOf(System.getProperty(MULE_TRANSPORT_FILE_SINGLEPOLLINSTANCE, "false")) || !synchronousProcessing;
+        this.notProcessingRecentlyFilesPeriod = Integer.valueOf(System.getProperty(MULE_NOT_PROCESSING_RECENTLY_FILES_PERIOD, "-1"));
         ObjectStoreManager objectStoreManager = getEndpoint().getMuleContext().getRegistry().get(MuleProperties.OBJECT_STORE_MANAGER);
-        filesBeingProcessingObjectStore = objectStoreManager.getObjectStore(getEndpoint().getName(),false,1000,60000,20000);
+        filesBeingProcessingObjectStore = objectStoreManager.getObjectStore(getEndpoint().getName(), false, 1000, 60000, 20000);
+        filesRecentlyProcessed = objectStoreManager.getObjectStore(getEndpoint().getName() + RECENTLY_PROCESSED_SUFFIX, false, -1, notProcessingRecentlyFilesPeriod, 20000);
     }
 
     @Override
@@ -213,13 +225,29 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
                         try
                         {
                             String fileAbsolutePath = file.getAbsolutePath();
+
+                            if (notProcessingRecentlyFilesPeriod != -1 && isFileRecentlyProcessed(fileAbsolutePath))
+                            {
+                                continue;
+                            }
+
+                            // Skipping empty files
+                            if (ignoreEmptyFiles && file.length() == 0)
+                            {
+                                if (logger.isDebugEnabled())
+                                {
+                                    logger.debug("Found empty file '" + file.getName() + "'. Skipping file.");
+                                }
+                                continue;
+                            }
+
                             try
                             {
                                 filesBeingProcessingObjectStore.store(fileAbsolutePath, fileAbsolutePath);
 
                                 if (logger.isDebugEnabled())
                                 {
-                                    logger.debug(String.format("Flag for '%s' stored successfully.", fileAbsolutePath));
+                                    logger.debug("Flag for '" + fileAbsolutePath + "' stored successfully.");
                                 }
                             }
                             catch (ObjectAlreadyExistsException e)
@@ -249,6 +277,26 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         }
     }
 
+    protected boolean isFileRecentlyProcessed(String fileAbsolutePath) throws ObjectStoreException
+    {
+        try
+        {
+            filesRecentlyProcessed.store(fileAbsolutePath, fileAbsolutePath);
+        }
+        catch (ObjectAlreadyExistsException e)
+        {
+            if (logger.isDebugEnabled())
+            {
+                logger.debug(String.format("File '%s' recently processed. Skipping file.", fileAbsolutePath));
+            }
+
+            return true;
+
+        }
+
+        return false;
+    }
+
     @Override
     protected boolean pollOnPrimaryInstanceOnly()
     {
@@ -260,7 +308,17 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         //TODO RM*: This can be put in a Filter. Also we can add an AndFileFilter/OrFileFilter to allow users to
         //combine file filters (since we can only pass a single filter to File.listFiles, we would need to wrap
         //the current And/Or filters to extend {@link FilenameFilter}
-        if (fileConnector.getCheckFileAge() && !isAgedFile(file, fileConnector.getFileAge()))
+        Long fileAge = null;
+        if (this.endpoint.getProperties().containsKey(PROPERTY_FILE_AGE))
+        {
+            fileAge = (Long) endpoint.getProperties().get(PROPERTY_FILE_AGE);
+        }
+        else
+        {
+            fileAge = fileConnector.getFileAge();
+        }
+
+        if (fileAge != null && !isAgedFile(file, fileAge))
         {
             removeProcessingMark(file.getAbsolutePath());
 
@@ -670,7 +728,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
      *
      * @return an array of files to be processed.
      * @throws org.mule.api.MuleException which will wrap any other exceptions or
-     *             errors.
+     *                                      errors.
      */
     List<File> listFiles() throws MuleException
     {
@@ -678,7 +736,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         {
             List<File> files = new ArrayList<File>();
             this.basicListFiles(readDirectory, files);
-            return (files.isEmpty() ? NO_FILES : files);
+            return (files.isEmpty() ? Collections.<File>emptyList() : files);
         }
         catch (Exception e)
         {
@@ -693,7 +751,18 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         // the listFiles calls above may actually return null (check the JDK code).
         if (files == null)
         {
-            return;
+            if (!readDirectory.exists())
+            {
+                throw new MuleRuntimeException(CoreMessages.createStaticMessage("Directory '%s' does not exist", currentDirectory.getPath()));
+            }
+            else if (!readDirectory.isDirectory())
+            {
+                throw new MuleRuntimeException(CoreMessages.createStaticMessage("'%s' is not a directory", currentDirectory.getPath()));
+            }
+            else
+            {
+                return;
+            }
         }
 
         for (File file : files)
@@ -758,8 +827,9 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
 
             Class<?> clazz = endpoint.getMuleContext().getExecutionClassLoader().loadClass(comparatorClassName.toString());
             Comparator<?> comparator = (Comparator<?>)clazz.newInstance();
-            return reverse ? new ReverseComparator(comparator) : comparator;
+            return reverse ? new ReverseComparator(comparator) : (Comparator<File>)comparator;
         }
         return null;
     }
+
 }
