@@ -31,7 +31,9 @@ import org.mule.util.IOUtils;
 import org.mule.util.ObjectUtils;
 import org.mule.util.StringUtils;
 
+import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -41,6 +43,7 @@ import java.util.Map;
 
 import javax.activation.DataHandler;
 import javax.activation.FileDataSource;
+import javax.activation.MimeType;
 import javax.activation.URLDataSource;
 
 import org.apache.commons.httpclient.HttpMethod;
@@ -60,8 +63,9 @@ import org.apache.commons.httpclient.methods.multipart.ByteArrayPartSource;
 import org.apache.commons.httpclient.methods.multipart.FilePart;
 import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
 import org.apache.commons.httpclient.methods.multipart.Part;
-import org.apache.commons.httpclient.methods.multipart.StringPart;
+import org.apache.commons.httpclient.methods.multipart.PartBase;
 import org.apache.commons.httpclient.params.HttpMethodParams;
+import org.apache.commons.httpclient.util.EncodingUtil;
 
 /**
  * <code>ObjectToHttpClientMethodRequest</code> transforms a MuleMessage into a
@@ -69,6 +73,9 @@ import org.apache.commons.httpclient.params.HttpMethodParams;
  */
 public class ObjectToHttpClientMethodRequest extends AbstractMessageTransformer
 {
+
+    public static final String CHARSET_PARAM_NAME = "charset";
+
     public ObjectToHttpClientMethodRequest()
     {
         setReturnDataType(DataTypeFactory.create(HttpMethod.class));
@@ -356,7 +363,7 @@ public class ObjectToHttpClientMethodRequest extends AbstractMessageTransformer
             }
 
             if (encoding != null && !"UTF-8".equals(encoding.toUpperCase())
-                && outboundMimeType.indexOf("charset") == -1)
+                && outboundMimeType.indexOf(CHARSET_PARAM_NAME) == -1)
             {
                 outboundMimeType += "; charset=" + encoding;
             }
@@ -482,7 +489,8 @@ public class ObjectToHttpClientMethodRequest extends AbstractMessageTransformer
             if (dh.getDataSource() instanceof StringDataSource)
             {
                 final StringDataSource ds = (StringDataSource) dh.getDataSource();
-                parts[i] = new StringPart(ds.getName(), IOUtils.toString(ds.getInputStream()));
+                final MimeType mimeType = new MimeType(ds.getContentType());
+                parts[i] = new CustomStringPart(ds.getName(), IOUtils.toString(ds.getInputStream()), mimeType.getParameter(CHARSET_PARAM_NAME), mimeType.getBaseType());
             }
             else
             {
@@ -520,4 +528,63 @@ public class ObjectToHttpClientMethodRequest extends AbstractMessageTransformer
 
         return new MultipartRequestEntity(parts, method.getParams());
     }
+
+    /**
+     * Replaces {@link org.apache.commons.httpclient.methods.multipart.StringPart} which does not let us
+     * change the content type of the part.
+     */
+    private static class CustomStringPart extends PartBase
+    {
+
+        public static final String DEFAULT_CONTENT_TYPE = "text/plain";
+        public static final String DEFAULT_CHARSET = "US-ASCII";
+        public static final String DEFAULT_TRANSFER_ENCODING = "8bit";
+
+        private byte[] content;
+        private String value;
+
+        public CustomStringPart(String name, String value, String charset, String contentType)
+        {
+            super(name, contentType, charset == null ? DEFAULT_CHARSET : charset, DEFAULT_TRANSFER_ENCODING);
+            if (value == null)
+            {
+                throw new IllegalArgumentException("Value may not be null");
+            }
+            else if (value.indexOf(0) != -1)
+            {
+                throw new IllegalArgumentException("NULs may not be present in string parts");
+            }
+            else
+            {
+                this.value = value;
+            }
+        }
+
+        private byte[] getContent()
+        {
+            if (this.content == null)
+            {
+                this.content = EncodingUtil.getBytes(this.value, this.getCharSet());
+            }
+
+            return this.content;
+        }
+
+        protected void sendData(OutputStream out) throws IOException
+        {
+            out.write(this.getContent());
+        }
+
+        protected long lengthOfData() throws IOException
+        {
+            return (long) this.getContent().length;
+        }
+
+        public void setCharSet(String charSet)
+        {
+            super.setCharSet(charSet);
+            this.content = null;
+        }
+    }
+
 }
