@@ -6,6 +6,11 @@
  */
 package org.mule.transport.jms;
 
+
+import static org.mule.api.config.MuleProperties.MULE_JMS_REDELIVERY_DELAY;
+import static org.mule.api.config.MuleProperties.MULE_JMS_MAX_REDELIVERY_DELAY;
+import static org.mule.api.config.MuleProperties.MULE_JMS_INITIAL_REDELIVERY_DELAY;
+
 import org.mule.api.Closeable;
 import org.mule.api.DefaultMuleException;
 import org.mule.api.MuleContext;
@@ -43,10 +48,14 @@ import org.mule.transport.jms.redelivery.RedeliveryHandlerFactory;
 import org.mule.util.BeanUtils;
 import org.mule.util.concurrent.ThreadNameHelper;
 
+import java.io.EOFException;
+import java.io.IOException;
+import java.io.InvalidObjectException;
 import java.text.MessageFormat;
 import java.util.Map;
 import java.util.Timer;
 import java.util.TimerTask;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.jms.Connection;
@@ -80,6 +89,16 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
     public static final int REDELIVERY_FAIL_ON_FIRST = 0;
 
     public static final int REDELIVERY_IGNORE = -1;
+    
+    public static final int PREFETCH_DEFAULT = -1;
+    
+    public static final int DEFAULT_MAX_REDELIVERY_DELAY = -1;
+    
+    public static final int DEFAULT_INITIAL_REDELIVERY_DELAY = -1;
+    
+    public static final int DEFAULT_REDELIVERY_DELAY = -1;
+
+    public static final String CONNECTION_STOPPING_ERROR_MESSAGE = "It is not possible to create a session since connection is being stopped.";
 
     private AtomicInteger receiverReportedExceptionCount = new AtomicInteger();
 
@@ -100,7 +119,15 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
     private boolean honorQosHeaders;
 
     private int maxRedelivery = REDELIVERY_FAIL_ON_FIRST;
+    
+    private int maxQueuePrefetch = PREFETCH_DEFAULT;
+    
+    private int maximumRedeliveryDelay = DEFAULT_MAX_REDELIVERY_DELAY;
+    
+    private int initialRedeliveryDelay  = DEFAULT_INITIAL_REDELIVERY_DELAY;
 
+    private int redeliveryDelay = DEFAULT_REDELIVERY_DELAY;
+    
     private boolean cacheJmsSessions = true;
 
     /**
@@ -108,6 +135,10 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
      */
     private boolean eagerConsumer = true;
 
+    private AtomicBoolean shouldRetryBrokerConnection = new AtomicBoolean(false);
+
+    private AtomicBoolean stillConnectingReceivers = new AtomicBoolean(false);
+
     ////////////////////////////////////////////////////////////////////////
     // JMS Connection
     ////////////////////////////////////////////////////////////////////////
@@ -190,6 +221,8 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
      */
     private volatile boolean disconnecting;
 
+    private volatile boolean stopping = false;
+
     private Timer responseTimeoutTimer;
 
     ////////////////////////////////////////////////////////////////////////
@@ -288,7 +321,7 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
     {
         // if an initial factory class was configured that takes precedence over the 
         // spring-configured connection factory or the one that our subclasses may provide
-        if (jndiInitialFactory != null || jndiNameResolver != null)
+        if (isConnectionFactoryRetrievedThroughJndi())
         {
             if (jndiNameResolver == null)
             {
@@ -339,6 +372,11 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         }
     }
 
+    private boolean isConnectionFactoryRetrievedThroughJndi()
+    {
+        return jndiInitialFactory != null || jndiNameResolver != null;
+    }
+
     private JndiNameResolver createDefaultJndiResolver()
     {
         if (logger.isDebugEnabled())
@@ -420,7 +458,13 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
 
     protected Connection createConnection() throws MuleException, JMSException
     {
-        if (connectionFactory == null)
+        // In case the connection factory is retrieved through Jndi,
+        // the connection factory should not be cached. There are cases where there are several
+        // failover URLs but each connection factory in the node only
+        // contains the URL for that node. In case of reconnection, it
+        // is necessary to attempt to retrieve the connection factory again
+        // using all the URLs in the jndiProviderUrl.
+        if (connectionFactory == null || isConnectionFactoryRetrievedThroughJndi())
         {
             try
             {
@@ -451,13 +495,22 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         }
 
         if (connection != null)
+        {
+            postCreationSetup(connection);
+        }
+        return connection;
+    }
+
+    protected void postCreationSetup(Connection connection) throws JMSException
+    {
+        try
         {
             // EE-1901: only sets the clientID if it was not already set
             if (clientId != null && !clientId.equals(connection.getClientID()))
             {
                 connection.setClientID(getClientId());
             }
-            // Only set the exceptionListener if one isn't already set.  This is because the CachingConnectionFactory
+            // Only set the exceptionListener if one isn't already set. This is because the CachingConnectionFactory
             // which may be in use doesn't permit exception strategy to be set, rather it is set on the ConnectionFactory
             // itself in this case.
             if (!embeddedMode && connection.getExceptionListener() == null)
@@ -465,7 +518,11 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
                 connection.setExceptionListener(this);
             }
         }
-        return connection;
+        catch (JMSException e)
+        {
+            silentlyCloseConnection(connection, connectionFactory);
+            throw e;
+        }
     }
 
     @Override
@@ -506,10 +563,20 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         }
     }
 
+
     @Override
     public void onException(JMSException jmsException)
     {
-        if (!disconnecting)
+        // In case the original exception cause is an IOException, it means something went
+        // wrong with the actual transport that connects to the broker. Since the receiver runs
+        // with a retryPolicy, the re-connection should be delegated to it.
+        if (jmsException.getCause() instanceof IOException && this.stillConnectingReceivers())
+        {
+            logger.error("The transport connecting to the JMS Broker failed. If a retry policy was configured, it should attempt to reconnect.");
+            shouldRetryBrokerConnection.set(true);
+        }
+
+        if (!disconnecting && !shouldRetryBrokerConnection.get())
         {
             Map<Object, MessageReceiver> receivers = getReceivers();
             boolean isMultiConsumerReceiver = false;
@@ -535,14 +602,21 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
 
             if (receiverReportedExceptionCount.incrementAndGet() >= expectedReceiverCount)
             {
+                prepareConnectorForConnectionException();
                 receiverReportedExceptionCount.set(0);
                 muleContext.getExceptionListener().handleException(new ConnectException(jmsException, this));
             }
         }
     }
 
+    protected void prepareConnectorForConnectionException()
+    {
+        // Override if necessary
+    }
+
     @Override
     protected void doConnect() throws Exception
+
     {
         connection = createConnection();
         if ((connectionFactoryProperties != null) && !connectionFactoryProperties.isEmpty())
@@ -560,10 +634,7 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
             {
                 // If connection throws an exception on start and connection is cached in ConnectionFactory then
                 // close/reset connection now.
-                if (connectionFactory instanceof Closeable)
-                {
-                    ((Closeable) connectionFactory).close();
-                }
+                silentlyCloseConnection(connection, connectionFactory);
                 throw e;
             }
         }
@@ -577,11 +648,7 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
             if (connection != null)
             {
                 disconnecting = true;
-                connection.close();
-                if (connectionFactory instanceof Closeable)
-                {
-                    ((Closeable) connectionFactory).close();
-                }
+                closeConnection(connection, connectionFactory);
             }
         }
         finally
@@ -591,6 +658,31 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         }
     }
 
+    protected void silentlyCloseConnection(Connection connection, ConnectionFactory connectionFactory)
+    {
+        try
+        {
+            closeConnection(connection, connectionFactory);
+        }
+        catch (Exception e)
+        {
+            logger.warn("Error on closing JMS connection: " + e.getMessage());
+        }
+    }
+    
+    private void closeConnection(Connection connection, ConnectionFactory connectionFactory) throws JMSException, MuleException
+    {
+        if (connection != null)
+        {
+            connection.close();
+        }
+
+        if (connectionFactory != null && connectionFactory instanceof Closeable)
+        {
+            ((Closeable) connectionFactory).close();
+        }
+    }
+
     @Override
     protected Object getReceiverKey(FlowConstruct flowConstruct, InboundEndpoint endpoint)
     {
@@ -671,9 +763,12 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
 
     private Session createSession(boolean transacted, boolean topic) throws JMSException
     {
-        Session session;
-        session = jmsSupport.createSession(connection, topic, transacted, acknowledgementMode, noLocal);
-        return session;
+        if(stopping)
+        {
+            throw new IllegalStateException(CONNECTION_STOPPING_ERROR_MESSAGE);
+        }
+
+        return jmsSupport.createSession(connection, topic, transacted, acknowledgementMode, noLocal);
     }
 
     @Override
@@ -712,13 +807,13 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         {
             if (logger.isDebugEnabled())
             {
-                logger.error("Closing non-TX session: " + session);
+                logger.debug("Closing non-TX session: " + session);
             }
             closeQuietly(session);
         }
         else if (logger.isDebugEnabled())
         {
-            logger.error("Not closing TX session: " + session);
+            logger.debug("Not closing TX session: " + session);
         }
     }
 
@@ -729,6 +824,7 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         {
             try
             {
+                stopping = true;
                 connection.stop();
             }
             catch (Exception e)
@@ -737,6 +833,10 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
                 // stop process should continue all the same
                 logger.warn("Jms connection failed to stop properly: ", e);
             }
+            finally
+            {
+                stopping = false;
+            }
         }
 
         if (jndiNameResolver != null)
@@ -1132,11 +1232,21 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
         return maxRedelivery;
     }
 
+    public int getMaxQueuePrefetch()
+    {
+        return maxQueuePrefetch;
+    }
+
     public void setMaxRedelivery(int maxRedelivery)
     {
         this.maxRedelivery = maxRedelivery;
     }
 
+    public void setMaxQueuePrefetch(int maxPrefetch)
+    {
+        this.maxQueuePrefetch = maxPrefetch;
+    }
+
     @Override
     public boolean isResponseEnabled()
     {
@@ -1484,4 +1594,68 @@ public class JmsConnector extends AbstractConnector implements ExceptionListener
     {
         responseTimeoutTimer.schedule(timerTask, timeout);
     }
+
+    public int getMaximumRedeliveryDelay()
+    {
+        String maximumRedeliveryDelay = System.getProperty(MULE_JMS_MAX_REDELIVERY_DELAY, "-1");
+        return Integer.parseInt(maximumRedeliveryDelay);
+    }
+
+    public void setMaximumRedeliveryDelay(int maximumRedeliveryDelay)
+    {
+        this.maximumRedeliveryDelay = maximumRedeliveryDelay;
+    }
+
+    public int getInitialRedeliveryDelay()
+    {
+        String initialRedeliveryDelay = System.getProperty(MULE_JMS_INITIAL_REDELIVERY_DELAY, "-1");
+        return Integer.parseInt(initialRedeliveryDelay);
+    }
+
+    public void setInitialRedeliveryDelay(int initialRedeliveryDelay)
+    {
+        this.initialRedeliveryDelay = initialRedeliveryDelay;
+    }
+
+    public int getRedeliveryDelay()
+    {
+        String redeliveryDelay = System.getProperty(MULE_JMS_REDELIVERY_DELAY, "-1");
+        return Integer.parseInt(redeliveryDelay);
+    }
+
+    public void setRedeliveryDelay(int redeliveryDelay)
+    {
+        this.redeliveryDelay = redeliveryDelay;
+    }
+
+    public boolean shouldRetryBrokerConnection()
+    {
+        return shouldRetryBrokerConnection.get();
+    }
+
+    public void setShouldRetryBrokerConnection(boolean aBoolean)
+    {
+        this.shouldRetryBrokerConnection.set(aBoolean);
+    }
+
+    public boolean stillConnectingReceivers()
+    {
+        return stillConnectingReceivers.get();
+    }
+
+    public void setStillConnectingReceivers(boolean aBoolean)
+    {
+        this.stillConnectingReceivers.set(aBoolean);
+    }
+
+    public boolean mustRecycleReceivers()
+    {
+        return true;
+    }
+
+    public boolean isStopping()
+    {
+        return stopping;
+    }
+
 }
