@@ -10,11 +10,11 @@ import org.mule.api.MuleEvent;
 import org.mule.api.MuleMessage;
 import org.mule.api.construct.FlowConstruct;
 import org.mule.api.endpoint.InboundEndpoint;
+import org.mule.api.execution.ExecutionCallback;
+import org.mule.api.execution.ExecutionTemplate;
 import org.mule.api.lifecycle.CreateException;
 import org.mule.api.transaction.Transaction;
 import org.mule.api.transport.Connector;
-import org.mule.api.execution.ExecutionCallback;
-import org.mule.api.execution.ExecutionTemplate;
 import org.mule.retry.policies.NoRetryPolicyTemplate;
 import org.mule.transaction.TransactionCoordination;
 import org.mule.transaction.XaTransaction;
@@ -27,6 +27,8 @@ import org.mule.util.MapUtils;
 
 import java.lang.reflect.UndeclaredThrowableException;
 import java.util.List;
+import java.util.Set;
+import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -42,11 +44,13 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
     public static final TimeUnit DEFAULT_JMS_POLL_TIMEUNIT = TimeUnit.MILLISECONDS;
 
     protected final JmsConnector connector;
+    private final long localTimeout;
     protected boolean reuseConsumer;
     protected boolean reuseSession;
     protected final ThreadContextLocal context = new ThreadContextLocal();
+    protected final Set<JmsThreadContext> allContexts = new CopyOnWriteArraySet<>();
     protected final long timeout;
-    private final AtomicReference<RedeliveryHandler> redeliveryHandler = new AtomicReference<RedeliveryHandler>();
+    private final AtomicReference<RedeliveryHandler> redeliveryHandler = new AtomicReference<>();
     private final boolean topic;
 
     @Override
@@ -87,7 +91,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
         // TODO AP: find appropriate value for polling frequency with the scheduler;
         // see setFrequency/setTimeUnit & VMMessageReceiver for more
         this.setTimeUnit(DEFAULT_JMS_POLL_TIMEUNIT);
-        this.setFrequency(MapUtils.getLongValue(endpoint.getProperties(), "pollingFrequency", DEFAULT_JMS_POLL_FREQUENCY));
+        this.setFrequency(DEFAULT_JMS_POLL_FREQUENCY);
 
         this.connector = (JmsConnector) connector;
         this.timeout = endpoint.getTransactionConfig().getTimeout();
@@ -127,6 +131,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
         // If we're using topics we don't want to use multiple receivers as we'll get
         // the same message multiple times
         this.setUseMultipleTransactedReceivers(!topic);
+        this.localTimeout = resolveReceiveTimeout();
     }
 
     @Override
@@ -170,15 +175,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
             {
                 try
                 {
-                    List messages = getMessages();
-                    if (messages != null && messages.size() > 0)
-                    {
-                        for (Object message : messages)
-                        {
-                            processMessage(message);
-                        }
-                    }
-                    return null;
+                    return doProcess();
                 }
                 catch (Exception e)
                 {
@@ -206,6 +203,37 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
                     throw e;
                 }
             }
+
+            protected MuleEvent doProcess() throws Exception
+            {
+                try
+                {
+                    List messages = getMessages();
+                    if (messages != null && messages.size() > 0)
+                    {
+                        for (Object message : messages)
+                        {
+                            processMessage(message);
+                        }
+                    }
+                    return null;
+                }
+                catch (JMSException e)
+                {
+                    throw new ConnectException(e, XaTransactedJmsMessageReceiver.this);
+                }
+                catch (Exception e)
+                {
+                    if (e.getCause() != null && e.getCause() instanceof JMSException)
+                    {
+                        throw new ConnectException(e.getCause(), XaTransactedJmsMessageReceiver.this);
+                    }
+                    else
+                    {
+                        throw e;
+                    }
+                }
+            }
         };
 
         processingCallback.execute(cb);
@@ -235,7 +263,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
         Message message = null;
         try
         {
-            message = consumer.receive(timeout);
+            message = consumer.receive(localTimeout);
         }
         catch (Exception e)
         {
@@ -287,6 +315,25 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
         return null;
     }
 
+    private long resolveReceiveTimeout()
+    {
+        long localTimeout = MapUtils.getLongValue(endpoint.getProperties(), "xaPollingTimeout", timeout);
+
+
+        if (localTimeout > timeout)
+        {
+            logger.warn(String.format("Transaction timeout ('%s') must be greater than the timeout used for polling messages ('%s'). Using transaction timeout", localTimeout, timeout));
+            localTimeout = timeout;
+        }
+
+        if (logger.isDebugEnabled())
+        {
+            logger.debug(String.format("Consumer is receiving in '%s' ith timeout '%d'", this, localTimeout));
+        }
+
+        return localTimeout;
+    }
+
     private void closeConsumerIfRequired(MessageConsumer consumer)
     {
         if (!this.reuseConsumer)
@@ -308,16 +355,26 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
     }
 
     /**
-     * Close Sesison and consumer
+     * Close Session and consumer
      */
     protected void closeResource(boolean force)
     {
         JmsThreadContext ctx = context.getContext();
-        if (ctx == null)
+        if (ctx.consumer == null && ctx.session == null)
         {
-            return;
+            for (JmsThreadContext threadContext : allContexts)
+            {
+                doCloseResource(force, threadContext);
+            }
         }
+        else
+        {
+            doCloseResource(force, ctx);
+        }
+    }
 
+    protected void doCloseResource(boolean force, JmsThreadContext ctx)
+    {
         // Close consumer
         if (force || !reuseSession || !reuseConsumer)
         {
@@ -347,10 +404,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
             JmsSupport jmsSupport = this.connector.getJmsSupport();
 
             JmsThreadContext ctx = context.getContext();
-            if (ctx == null)
-            {
-                ctx = new JmsThreadContext();
-            }
+            allContexts.add(context.getContext());
 
             Session session;
             Transaction tx = TransactionCoordination.getInstance().getTransaction();
@@ -375,10 +429,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
                 }
             }
 
-            if (reuseSession)
-            {
-                ctx.session = session;
-            }
+            ctx.session = session;
 
             // TODO How can I verify that the consumer is active?
             if (this.reuseConsumer && ctx.consumer != null)
@@ -422,10 +473,7 @@ public class XaTransactedJmsMessageReceiver extends TransactedPollingMessageRece
             // Create consumer
             MessageConsumer consumer = jmsSupport.createConsumer(session, dest, selector, connector.isNoLocal(),
                 durableName, topic, endpoint);
-            if (reuseConsumer)
-            {
-                ctx.consumer = consumer;
-            }
+            ctx.consumer = consumer;
             return consumer;
         }
         catch (JMSException e)
