@@ -96,6 +96,12 @@ public class SftpMessageReceiver extends AbstractPollingMessageReceiver
             }
             catch (Exception e)
             {
+                if (!connected.get())
+                {
+                    logger.debug("No connection exception will be raised as it is already disconnected");  
+                    return;
+                }
+                
                 connected.set(false);
                 throw new ConnectException(e, this);
             }
@@ -147,7 +153,6 @@ public class SftpMessageReceiver extends AbstractPollingMessageReceiver
         catch (Exception e)
         {
             logger.error("Error in poll", e);
-            getEndpoint().getMuleContext().getExceptionListener().handleException(e);
             throw e;
         }
     }
