@@ -1,13 +1,9 @@
 /*
- * $Id: ApplicationStartupSplashScreen.java 20847 2010-12-29 20:17:22Z aperepel $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.util;
 
 import org.mule.api.MuleContext;
@@ -24,7 +20,7 @@ public class ApplicationStartupSplashScreen extends SplashScreen
     {
         header.add("Application: " + context.getConfiguration().getId());
         header.add(String.format("OS encoding: %s, Mule encoding: %s",
-                                 System.getProperty("file.encoding"),
+                                 SystemUtils.FILE_SEPARATOR,
                                  context.getConfiguration().getDefaultEncoding()));
         header.add(" ");
     }
