@@ -1,15 +1,12 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.util;
 
+import static org.mule.util.FileUtils.deleteFile;
 import org.mule.module.boot.MuleBootstrapUtils;
 
 import java.io.ByteArrayOutputStream;
@@ -81,7 +78,7 @@ public final class JarUtils
             combinedEntries.putAll(entries);
             File tmpJarFile = File.createTempFile(jarFile.getName(), null);
             createJarFileEntries(tmpJarFile, combinedEntries);
-            jarFile.delete();
+            deleteFile(jarFile);
             FileUtils.renameFile(tmpJarFile, jarFile);
         }
     }
